@@ -1,9 +1,9 @@
 import typing
 import warnings
 
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 class TypeAnnotation:
@@ -186,8 +186,6 @@ class SpecializationContainer:
             # Store values in temporary variables
             bytecode = [
                 Instruction(
-                    self.target,
-                    -1,
                     Opcodes.STORE_FAST,
                     f"&fast_tmp_specialize_local::{i}",
                 )
@@ -196,11 +194,9 @@ class SpecializationContainer:
 
             bytecode += [
                 (
-                    instruction.update_owner(self.target, -1)
+                    instruction
                     if isinstance(instruction, Instruction)
                     else Instruction(
-                        self.target,
-                        -1,
                         Opcodes.LOAD_FAST,
                         f"&fast_tmp_specialize_local::{instruction.arg_id}",
                     )
@@ -261,18 +257,12 @@ class SpecializationContainer:
                 self.method_call_descriptor.lookup_method_instr.change_arg_value(
                     self.invoke_before
                 )
-                self.method_call_descriptor.call_method_instr.change_arg(arg_count)
+                call = self.method_call_descriptor.call_method_instr
+                call.change_arg(arg_count)
 
-                self.method_call_descriptor.call_method_instr.insert_after(
-                    Instruction(
-                        self.underlying_function, -1, opcode_or_name=Opcodes.POP_TOP
-                    ),
-                    Instruction(
-                        self.underlying_function,
-                        -1,
-                        opcode_or_name=Opcodes.LOAD_CONST,
-                        arg_value=self.constant_value[0],
-                    ),
+                call.insert_after(
+                    Instruction.create_with_same_info(call, Opcodes.POP_TOP),
+                    Instruction.create_with_same_info(call, Opcodes.LOAD_CONST, self.constant_value[0]),
                 )
 
             else:
