@@ -4,7 +4,7 @@ from abc import ABC
 
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
 from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 
@@ -48,6 +48,11 @@ class ParsingScope:
         self.module_file: str = None
         self.last_base_token: AbstractToken = None
         self.macro_parameter_namespace: typing.Dict[str] = {}
+        self.filled_locals = set()
+
+        self.current_macro_assembly = None
+
+        self.closure_locals: typing.Set[str] = set()
 
     def scope_name_generator(self, suffix="") -> str:
         name = f"%INTERNAL:{self._name_counter}"
@@ -113,6 +118,7 @@ class ParsingScope:
         sub_scope_name: str | list = None,
         copy_labels=False,
         keep_scope_name_generator: bool = None,
+        shared_locals=True,
     ):
         instance = ParsingScope()
         if copy_labels:
@@ -131,6 +137,9 @@ class ParsingScope:
         elif keep_scope_name_generator:
             instance.scope_name_generator = self.scope_name_generator
 
+        if shared_locals:
+            instance.filled_locals = self.filled_locals
+
         return instance
 
     def insert_into_scope(
@@ -193,6 +202,12 @@ class AbstractSourceExpression(AbstractExpression, IAssemblyStructureVisitable,
     ):
         pass
 
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
+        raise NotImplementedError
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return tuple()
+
 
 class AbstractAccessExpression(AbstractSourceExpression, ABC):
     PREFIX: str | None = None
@@ -236,6 +251,9 @@ class IIdentifierAccessor:
     def __call__(self, scope: ParsingScope):
         raise NotImplementedError
 
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return tuple()
+
 
 class MacroExpandedIdentifier(IIdentifierAccessor):
     def __init__(
@@ -246,6 +264,9 @@ class MacroExpandedIdentifier(IIdentifierAccessor):
         self.macro_name = macro_name
         self.token = token
 
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return self.token
+
     def __hash__(self):
         return hash(self.macro_name)
 
@@ -259,7 +280,7 @@ class MacroExpandedIdentifier(IIdentifierAccessor):
 
     def __call__(self, scope: ParsingScope):
         from bytecodemanipulation.assembler.syntax_errors import (
-            throw_positioned_syntax_error,
+            throw_positioned_error,
         )
         from bytecodemanipulation.data.shared.expressions.ConstantAccessExpression import (
             ConstantAccessExpression,
@@ -278,7 +299,7 @@ class MacroExpandedIdentifier(IIdentifierAccessor):
             raise SyntaxError("no scope provided")
 
         if self.token[1].text not in scope.macro_parameter_namespace:
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope,
                 self.token,
                 "Could not find name in macro parameter space",
@@ -288,7 +309,7 @@ class MacroExpandedIdentifier(IIdentifierAccessor):
 
         if isinstance(value, ConstantAccessExpression):
             if not isinstance(value.value, str):
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     self.token,
                     f"Expected 'string' for name de-referencing, got {value}",
@@ -306,7 +327,7 @@ class MacroExpandedIdentifier(IIdentifierAccessor):
         ):
             return value.get_name(scope)
 
-        raise throw_positioned_syntax_error(
+        raise throw_positioned_error(
             scope,
             self.token,
             f"Expected <static evaluated expression> for getting the name for storing, got {value}",
@@ -314,8 +335,9 @@ class MacroExpandedIdentifier(IIdentifierAccessor):
 
 
 class StaticIdentifier(IIdentifierAccessor):
-    def __init__(self, name: str):
+    def __init__(self, name: str, token: AbstractToken = None):
         self.name = name
+        self.token = token
 
     def __eq__(self, other):
         return (
@@ -326,7 +348,10 @@ class StaticIdentifier(IIdentifierAccessor):
         return hash(self.name)
 
     def __repr__(self):
-        return repr(self.name)
+        return "!" + repr(self.name)
 
     def __call__(self, scope: ParsingScope):
         return self.name
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (self.token,)
