@@ -8,7 +8,7 @@ from bytecodemanipulation.assembler.AbstractBase import (
 )
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.syntax_errors import _syntax_wrapper
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.data.shared.instructions.AbstractInstruction import (
     AbstractAssemblyInstruction,
 )
@@ -60,7 +60,7 @@ from bytecodemanipulation.data.shared.expressions.TopOfStackAccessExpression imp
 )
 from bytecodemanipulation.data.shared.instructions.OpAssembly import AbstractOpAssembly
 
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 
 from bytecodemanipulation.assembler.Lexer import (
     Lexer,
@@ -111,7 +111,10 @@ except ImportError:
 
 def create_instruction(token: AbstractToken, *args, **kwargs) -> Instruction:
     instr = Instruction(*args, **kwargs)
-    instr.source_location = token.line, token.column, token.span
+
+    if token is not None:
+        instr.source_location = token.line, token.column, token.span
+
     return instr
 
 
@@ -176,7 +179,7 @@ class Parser(AbstractParser):
                 scope.scope_path += namespace_part
 
         if not self.try_consume(SpecialToken("{")):
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope, self.try_inspect(), "expected '{'"
             )
 
@@ -211,7 +214,7 @@ class Parser(AbstractParser):
                 continue
 
             if not (instr_token := self.try_consume(IdentifierToken)):
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope, self.try_inspect(), "expected identifier"
                 )
 
@@ -220,7 +223,7 @@ class Parser(AbstractParser):
 
             if instr_token.text not in self.INSTRUCTIONS:
                 if not (instr := self.try_parse_custom_assembly(instr_token, scope)):
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope,
                         instr_token,
                         "expected <assembly instruction name> or <assembly macro name>",
@@ -251,13 +254,13 @@ class Parser(AbstractParser):
                 if eof_error and self.try_inspect() is None:
                     print(self.try_inspect())
                     print(repr(root))
-                    raise throw_positioned_syntax_error(scope, self[-1], eof_error)
+                    raise throw_positioned_error(scope, self[-1], eof_error)
 
                 break
 
             print(self[-1].line, self[-1], expr.line)
 
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope,
                 self.try_inspect(),
                 "Expected <newline> or ';' after assembly instruction",
@@ -356,11 +359,38 @@ class Parser(AbstractParser):
 
         elif start_token.text == "$":
             self.consume(SpecialToken("$"), err_arg=scope)
-            expr = LocalAccessExpression(self.parse_identifier_like(scope), start_token)
+
+            prefix = ""
+
+            while self.try_consume(SpecialToken("|")):
+                prefix += "|"
+
+            if prefix == "":
+                while self.try_consume(SpecialToken(":")):
+                    prefix += ":"
+
+            elif error := self.try_consume(SpecialToken(":")):
+                raise throw_positioned_error(
+                    scope,
+                    error,
+                    "Cannot parse '|' and ':' for <local variable name>",
+                )
+
+            expr = LocalAccessExpression(self.parse_identifier_like(scope), start_token, prefix=prefix)
 
         elif start_token.text == "§":
             self.consume(SpecialToken("§"), err_arg=scope)
-            expr = DerefAccessExpression(self.parse_identifier_like(scope), start_token)
+
+            identifier = self.try_parse_identifier_like()
+
+            if identifier is None:
+                raise throw_positioned_error(
+                    scope,
+                    [start_token, self[0]],
+                    "Expected <identifier-like> after '§' for cell-var reference",
+                )
+
+            expr = DerefAccessExpression(identifier, start_token)
 
         elif start_token.text == "&":
             self.consume(SpecialToken("&"), err_arg=scope)
@@ -397,7 +427,7 @@ class Parser(AbstractParser):
             self.consume(start_token, err_arg=scope)
 
             if not (opening := self.try_consume(SpecialToken("("))):
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     self[-1:1],
                     "expected '(' after OP when used in expressions",
@@ -406,7 +436,7 @@ class Parser(AbstractParser):
             expr = AbstractOpAssembly.IMPLEMENTATION.consume(self, scope)
 
             if not self.try_consume(SpecialToken(")")):
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     [opening, self[0]],
                     "expected ')' after operation",
@@ -427,7 +457,7 @@ class Parser(AbstractParser):
                             scope=scope,
                         )
                     ):
-                        raise throw_positioned_syntax_error(
+                        raise throw_positioned_error(
                             scope,
                             self.try_inspect() or self[-1],
                             "expected <expression>"
@@ -439,7 +469,7 @@ class Parser(AbstractParser):
                         )
 
                     if not self.try_consume(SpecialToken("]")):
-                        raise throw_positioned_syntax_error(
+                        raise throw_positioned_error(
                             scope, self.try_inspect() or self[-1], "expected ']''"
                         )
 
@@ -459,7 +489,7 @@ class Parser(AbstractParser):
                                 allow_calls=allow_calls,
                             )
                         ):
-                            raise throw_positioned_syntax_error(
+                            raise throw_positioned_error(
                                 scope,
                                 self.try_inspect() or self[-1],
                                 "expected <expression>"
@@ -473,7 +503,7 @@ class Parser(AbstractParser):
                             )
 
                         if not self.try_consume(SpecialToken(")")):
-                            raise throw_positioned_syntax_error(
+                            raise throw_positioned_error(
                                 scope,
                                 [opening_bracket, self.try_inspect()],
                                 "expected ')'",
@@ -547,16 +577,49 @@ class Parser(AbstractParser):
             return MacroExpandedIdentifier(expr[1].text, expr)
 
         if expr := self.try_consume(IdentifierToken):
-            return StaticIdentifier(expr.text)
+            return StaticIdentifier(expr.text, expr)
 
     def parse_identifier_like(self, scope: ParsingScope) -> IIdentifierAccessor:
         identifier = self.try_parse_identifier_like()
 
         if identifier is None:
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope,
                 self[0],
                 "expected <identifier> or &<identifier>",
             )
 
         return identifier
+
+    def try_parse_jump_target(self) -> typing.List[IIdentifierAccessor] | None:
+        self.save()
+        tokens = []
+
+        if not (t := self.try_parse_identifier_like()):
+            return
+
+        tokens.append(t)
+
+        while self.try_consume(SpecialToken(":")):
+            t = self.try_parse_identifier_like()
+
+            if t is None:
+                self.rollback()
+                return
+
+            tokens.append(t)
+
+        self.discard_save()
+        return tokens
+
+    def parse_jump_target(self, scope: ParsingScope) -> typing.List[IIdentifierAccessor]:
+        tokens = self.try_parse_jump_target()
+
+        if tokens is None:
+            raise throw_positioned_error(
+                scope,
+                self[0],
+                "expected <identifier-like>[{':' <identifier like>}] for jump target"
+            )
+
+        return tokens
