@@ -50,17 +50,19 @@ def _print_complex_token_location(
             print(f'File "{file}", line {line + 1}', file=sys.stderr)
         previous_line_no = line
 
+        if line >= len(content):
+            continue
+
         print(content[line].removesuffix("\n"), file=sys.stderr)
         print(error_location, file=sys.stderr)
 
 
-def throw_positioned_syntax_error(
+def throw_positioned_error(
     scope: ParsingScope,
     token: AbstractToken | typing.List[AbstractToken | None] | None,
     message: str,
     exc_type: typing.Type[Exception] = SyntaxError,
 ) -> Exception:
-    print("at", token, file=sys.stderr)
     if scope and scope.module_file and token:
         if isinstance(token, list):
             _print_complex_token_location(scope, token, exc_type=exc_type)
@@ -84,5 +86,32 @@ def throw_positioned_syntax_error(
     return exc_type(message)
 
 
+def print_positional_warning(
+    scope: ParsingScope,
+    token: AbstractToken | typing.List[AbstractToken | None] | None,
+    message: str,
+    warning_type: typing.Type[Exception] = SyntaxWarning,
+):
+    if scope and scope.module_file and token:
+        if isinstance(token, list):
+            _print_complex_token_location(scope, token, exc_type=warning_type)
+        else:
+            file = scope.module_file.replace("\\", "/")
+            print(f'File "{file}", line {token.line + 1}', file=sys.stderr)
+            with open(scope.module_file, mode="r", encoding="utf-8") as f:
+                content = f.readlines()
+
+            line = content[token.line]
+            print(line.rstrip(), file=sys.stderr)
+            print(
+                (" " * token.column) + "^" + ("~" * (token.span - 1)), file=sys.stderr
+            )
+
+        print(f"-> {warning_type.__name__}: {message}", file=sys.stderr)
+    else:
+        # print(scope, scope.module_file if scope else None, token)
+        print(f"{warning_type.__name__}: @{token}: {message}")
+
+
 def _syntax_wrapper(token, text, scope):
-    raise throw_positioned_syntax_error(scope, token, text)
+    raise throw_positioned_error(scope, token, text)
