@@ -1,10 +1,9 @@
 import typing
 import random
 
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 from bytecodemanipulation.Specialization import SpecializationContainer, register
-from bytecodemanipulation.MutableFunction import Instruction
-
+from bytecodemanipulation.opcodes.Instruction import Instruction
 
 ASSERT_TYPE_CASTS = False
 DISCARD_ALL_ANY_WITHOUT_SIDE_EFFECT_CHECK = True
@@ -16,37 +15,37 @@ def specialize_typing_cast(container: SpecializationContainer):
     data_type, value = container.get_arg_specifications()
 
     if ASSERT_TYPE_CASTS:
-        nop = Instruction(container.target, -1, Opcodes.NOP)
+        nop = Instruction(Opcodes.NOP)
 
         bytecode = [
             value,
-            Instruction(container.target, -1, Opcodes.DUP_TOP),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, isinstance),
-            Instruction(container.target, -1, Opcodes.ROT_TWO),
+            Instruction(Opcodes.DUP_TOP),
+            Instruction(Opcodes.LOAD_CONST, isinstance),
+            Instruction(Opcodes.ROT_TWO),
             data_type,
-            Instruction(container.target, -1, Opcodes.CALL_FUNCTION, arg=2),
-            Instruction(container.target, -1, Opcodes.POP_JUMP_IF_TRUE, nop),
+            Instruction(Opcodes.CALL_FUNCTION, arg=2),
+            Instruction(Opcodes.POP_JUMP_IF_TRUE, nop),
             Instruction(
-                container.target, -1, Opcodes.LOAD_CONST, "expected data type '"
+                Opcodes.LOAD_CONST, "expected data type '"
             ),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, repr),
+            Instruction(Opcodes.LOAD_CONST, repr),
             data_type,
-            Instruction(container.target, -1, Opcodes.CALL_FUNCTION, arg=1),
-            Instruction(container.target, -1, Opcodes.BINARY_ADD),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, "', but got '"),
-            Instruction(container.target, -1, Opcodes.BINARY_ADD),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, repr),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, type),
+            Instruction(Opcodes.CALL_FUNCTION, arg=1),
+            Instruction(Opcodes.BINARY_ADD),
+            Instruction(Opcodes.LOAD_CONST, "', but got '"),
+            Instruction(Opcodes.BINARY_ADD),
+            Instruction(Opcodes.LOAD_CONST, repr),
+            Instruction(Opcodes.LOAD_CONST, type),
             value,
-            Instruction(container.target, -1, Opcodes.CALL_FUNCTION, arg=1),
-            Instruction(container.target, -1, Opcodes.CALL_FUNCTION, arg=1),
-            Instruction(container.target, -1, Opcodes.BINARY_ADD),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, "'"),
-            Instruction(container.target, -1, Opcodes.BINARY_ADD),
-            Instruction(container.target, -1, Opcodes.LOAD_CONST, ValueError),
-            Instruction(container.target, -1, Opcodes.ROT_TWO),
-            Instruction(container.target, -1, Opcodes.CALL_FUNCTION, arg=1),
-            Instruction(container.target, -1, Opcodes.RAISE_VARARGS, arg=1),
+            Instruction(Opcodes.CALL_FUNCTION, arg=1),
+            Instruction(Opcodes.CALL_FUNCTION, arg=1),
+            Instruction(Opcodes.BINARY_ADD),
+            Instruction(Opcodes.LOAD_CONST, "'"),
+            Instruction(Opcodes.BINARY_ADD),
+            Instruction(Opcodes.LOAD_CONST, ValueError),
+            Instruction(Opcodes.ROT_TWO),
+            Instruction(Opcodes.CALL_FUNCTION, arg=1),
+            Instruction(Opcodes.RAISE_VARARGS, arg=1),
             nop,
         ]
 
@@ -433,22 +432,43 @@ def specialize_any(container: SpecializationContainer):
 
 
 @register(sum)
-def specialize_any(container: SpecializationContainer):
+def specialize_sum(container: SpecializationContainer):
     args = container.get_arg_specifications()
 
     if len(args) == 1:
         if args[0].get_normalized_data_instr().opcode == Opcodes.LOAD_CONST:
-            container.replace_with_constant_value(
-                sum(args[0].get_normalized_data_instr().arg_value)
-            )
+            obj = args[0].get_normalized_data_instr().arg_value
+
+            if None in obj:
+                container.replace_with_raise_exception(
+                    TypeError(
+                        "TypeError: unsupported operand type(s) for +: 'int' and 'NoneType'"
+                    )
+                )
+                return
+
+            container.replace_with_constant_value(sum(obj))
+
         elif args[0].get_normalized_data_instr().opcode in (
             Opcodes.BUILD_LIST,
             Opcodes.BUILD_TUPLE,
             Opcodes.BUILD_SET,
         ):
+            for source in [
+                next(args[0].get_normalized_data_instr().trace_stack_position(i))
+                for i in range(args[0].get_normalized_data_instr().arg)
+            ]:
+                if source.opcode == Opcodes.LOAD_CONST and source.arg_value is None:
+                    container.replace_with_raise_exception(
+                        TypeError(
+                            "TypeError: unsupported operand type(s) for +: 'int' and 'NoneType'"
+                        )
+                    )
+                    return
+
             count = args[0].get_normalized_data_instr().arg
             args[0].get_normalized_data_instr().change_opcode(Opcodes.NOP)
             container.replace_call_with_opcodes(
-                [Instruction(None, -1, Opcodes.BINARY_ADD) for _ in range(count - 1)],
+                [Instruction(Opcodes.BINARY_ADD) for _ in range(count - 1)],
                 leave_args_on_stack=True,
             )
