@@ -4,22 +4,20 @@ from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import IAssemblyStructureVisitable
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
 from bytecodemanipulation.assembler.util.tokenizer import IdentifierToken
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.assembler.AbstractBase import IIdentifierAccessor, StaticIdentifier
+from bytecodemanipulation.assembler.AbstractBase import (
+    IIdentifierAccessor,
+    StaticIdentifier,
+)
 
 
 class AttributeAccessExpression(AbstractAccessExpression):
-    def __init__(
-        self, root: AbstractAccessExpression, name: IIdentifierAccessor | str
-    ):
+    def __init__(self, root: AbstractAccessExpression, name: IIdentifierAccessor | str):
         self.root = root
-        self.name = (
-            name
-            if not isinstance(name, str)
-            else StaticIdentifier(name)
-        )
+        self.name = name if not isinstance(name, str) else StaticIdentifier(name)
 
     def __eq__(self, other):
         return (
@@ -38,18 +36,14 @@ class AttributeAccessExpression(AbstractAccessExpression):
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
         return self.root.emit_bytecodes(function, scope) + [
-            Instruction(
-                function, -1, "LOAD_ATTR", self.name(scope)
-            )
+            Instruction("LOAD_ATTR", self.name(scope))
         ]
 
     def emit_store_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
         return self.root.emit_bytecodes(function, scope) + [
-            Instruction(
-                function, -1, "STORE_ATTR", self.name(scope)
-            )
+            Instruction("STORE_ATTR", self.name(scope))
         ]
 
     def visit_parts(
@@ -63,3 +57,17 @@ class AttributeAccessExpression(AbstractAccessExpression):
         return visitor(
             self, (self.root.visit_parts(visitor, parents + [self]),), parents
         )
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (
+            list(self.root.get_tokens()) + list(self.name.get_tokens()) + [self.token]
+        )
+
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
+        base = self.root.evaluate_static_value(scope)
+        name = self.name(scope)
+
+        if not hasattr(base, name):
+            raise NotImplementedError
+
+        return getattr(base, name)
