@@ -3,9 +3,9 @@ import typing
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 
@@ -28,11 +28,17 @@ class ConstantAccessExpression(AbstractAccessExpression):
     def emit_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
-        return [Instruction(function, -1, "LOAD_CONST", self.value)]
+        return [Instruction("LOAD_CONST", self.value)]
 
     def emit_store_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
-        raise throw_positioned_syntax_error(
+        raise throw_positioned_error(
             scope, self.token, f"Cannot assign to a constant: {self}"
         )
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (self.token,)
+
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
+        return self.value
