@@ -2,8 +2,10 @@ import typing
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 class DerefAccessExpression(AbstractAccessExpression):
@@ -19,7 +21,7 @@ class DerefAccessExpression(AbstractAccessExpression):
 
         return [
             Instruction.create_with_token(
-                self.token, function, -1, "LOAD_DEREF", value, _decode_next=False
+                self.token, Opcodes.LOAD_DEREF, value
             )
         ]
 
@@ -33,6 +35,12 @@ class DerefAccessExpression(AbstractAccessExpression):
 
         return [
             Instruction.create_with_token(
-                self.token, function, -1, "STORE_DEREF", value
+                self.token, Opcodes.STORE_DEREF, value
             )
         ]
+
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
+        raise NotImplementedError  # todo: implement in some cases
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (self.token,)
