@@ -2,9 +2,10 @@ import typing
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 class DiscardValueExpression(AbstractAccessExpression):
@@ -19,4 +20,7 @@ class DiscardValueExpression(AbstractAccessExpression):
     def emit_store_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
-        return [Instruction(function, -1, Opcodes.POP_TOP)]
+        return [Instruction(Opcodes.POP_TOP)]
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (self.token,)
