@@ -3,9 +3,11 @@ import typing
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 class GlobalStaticAccessExpression(AbstractAccessExpression):
@@ -23,7 +25,7 @@ class GlobalStaticAccessExpression(AbstractAccessExpression):
             value = global_dict.get(key)
 
         return [
-            Instruction.create_with_token(self.name, function, -1, "LOAD_CONST", value)
+            Instruction.create_with_token(tuple(self.name.get_tokens())[0], Opcodes.LOAD_CONST, value)
         ]
 
     def emit_store_bytecodes(
@@ -31,13 +33,16 @@ class GlobalStaticAccessExpression(AbstractAccessExpression):
     ) -> typing.List[Instruction]:
         raise RuntimeError("Cannot assign to a constant global")
 
-    def get_static_value(self, scope: ParsingScope) -> typing.Any:
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
         if self.get_name(scope) in scope.globals_dict:
             return scope.globals_dict[self.get_name(scope)]
 
-        raise throw_positioned_syntax_error(
+        raise throw_positioned_error(
             scope,
             self.token,
             f"Name {self.get_name(scope)} not found!",
             NameError,
         )
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (self.token,)
