@@ -1,12 +1,30 @@
 import builtins
+import os
+import sys
 import types
 import typing
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
+
+
+root = __file__
+for _ in range(5):
+    root = os.path.dirname(root)
+
+
+with open(root+"/setup.py", mode="r") as f :
+    line = list(f.readlines())[8].strip()
+
+    if not line.startswith("version="):
+        raise RuntimeError("could not find version information")
+
+    parts = list(map(int, line.removeprefix("version=\"").removesuffix("\",").split(".")))
+    BCM_VERSION = parts[0] * 10000 + parts[1] * 100 + parts[2]
 
 
 class ModuleAccessExpression(AbstractAccessExpression):
@@ -14,6 +32,11 @@ class ModuleAccessExpression(AbstractAccessExpression):
     PREFIX = "~"
     _CACHE = builtins.__dict__.copy()
 
+    _CACHE.update({
+        "PY_VERSION": sys.version_info.major * 100 + sys.version_info.minor,
+        "BCM_VERSION": BCM_VERSION,
+    })
+
     @classmethod
     def _cached_lookup(cls, module_name: str) -> types.ModuleType:
         if module_name not in cls._CACHE:
@@ -28,7 +51,7 @@ class ModuleAccessExpression(AbstractAccessExpression):
         value = self._cached_lookup(self.get_name(scope))
         return [
             Instruction.create_with_token(
-                self.token, function, -1, Opcodes.LOAD_CONST, value
+                self.token, Opcodes.LOAD_CONST, value
             )
         ]
 
@@ -37,5 +60,8 @@ class ModuleAccessExpression(AbstractAccessExpression):
     ) -> typing.List[Instruction]:
         raise RuntimeError
 
-    def get_static_value(self, scope: ParsingScope) -> typing.Any:
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
         return self._cached_lookup(self.get_name(scope))
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return (self.token,)
