@@ -4,24 +4,26 @@ from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import IAssemblyStructureVisitable
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
-from bytecodemanipulation.assembler.util.tokenizer import IdentifierToken
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.assembler.AbstractBase import IIdentifierAccessor, StaticIdentifier
+from bytecodemanipulation.assembler.AbstractBase import (
+    IIdentifierAccessor,
+    StaticIdentifier,
+)
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 class StaticAttributeAccessExpression(AbstractAccessExpression):
     IS_STATIC = True
 
     def __init__(
-        self, root: AbstractAccessExpression, name: typing.Union["IIdentifierAccessor", str],
+        self,
+        root: AbstractAccessExpression,
+        name: typing.Union["IIdentifierAccessor", str],
     ):
         self.root = root
-        self.name = (
-            name
-            if not isinstance(name, str)
-            else StaticIdentifier(name)
-        )
+        self.name = name if not isinstance(name, str) else StaticIdentifier(name)
 
     def __eq__(self, other):
         return (
@@ -41,10 +43,8 @@ class StaticAttributeAccessExpression(AbstractAccessExpression):
     ) -> typing.List[Instruction]:
         return self.root.emit_bytecodes(function, scope) + [
             Instruction.create_with_token(
-                self.name(scope),
-                function,
-                -1,
-                "STATIC_ATTRIBUTE_ACCESS",
+                tuple(self.name.get_tokens())[0],
+                Opcodes.STATIC_ATTRIBUTE_ACCESS,
                 self.name(scope),
             )
         ]
@@ -65,3 +65,14 @@ class StaticAttributeAccessExpression(AbstractAccessExpression):
         return visitor(
             self, (self.root.visit_parts(visitor, parents + [self]),), parents
         )
+
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
+        base = self.root.evaluate_static_value(scope)
+
+        if not hasattr(base, self.name(scope)):
+            raise NotImplementedError
+
+        return getattr(base, self.name(scope))
+
+    def get_tokens(self) -> typing.Iterable[AbstractToken]:
+        return list(self.root.get_tokens()) + [self.token]
