@@ -2,20 +2,22 @@ import typing
 from abc import ABC
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
+from bytecodemanipulation.assembler.AbstractBase import IIdentifierAccessor, StaticIdentifier
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
+from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
 from bytecodemanipulation.data.shared.instructions.AbstractInstruction import (
     AbstractAssemblyInstruction,
 )
 from bytecodemanipulation.assembler.util.tokenizer import IdentifierToken
 from bytecodemanipulation.assembler.AbstractBase import IAssemblyStructureVisitable
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.assembler.AbstractBase import AbstractSourceExpression
 from bytecodemanipulation.assembler.Lexer import SpecialToken
 from bytecodemanipulation.data.shared.expressions.MacroAccessExpression import (
     MacroAccessExpression,
 )
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
 
 
@@ -37,11 +39,13 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
 
         def __init__(
             self,
-            source: typing.Union["AbstractAccessExpression", IdentifierToken],
+            source: typing.Union["AbstractAccessExpression", IIdentifierAccessor],
             is_dynamic: bool = False,
+            token: AbstractToken =None,
         ):
             self.source = source
             self.is_dynamic = is_dynamic
+            self.token = token
 
         def __repr__(self):
             return f"{type(self).__name__}{'' if not self.is_dynamic else 'Dynamic'}({self.source})"
@@ -92,15 +96,16 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
 
         def __init__(
             self,
-            key: IdentifierToken | str,
+            key: IIdentifierAccessor | str,
             source: typing.Union["AbstractAccessExpression", IdentifierToken],
             is_dynamic: bool = False,
+            token: AbstractToken = None,
         ):
-            self.key = key if isinstance(key, IdentifierToken) else IdentifierToken(key)
-            super().__init__(source, is_dynamic=is_dynamic)
+            self.key = key if not isinstance(key, str) else StaticIdentifier(key)
+            super().__init__(source, is_dynamic=is_dynamic, token=token)
 
         def __repr__(self):
-            return f"{type(self).__name__}{'' if not self.is_dynamic else 'Dynamic'}({self.key.text} = {self.source})"
+            return f"{type(self).__name__}{'' if not self.is_dynamic else 'Dynamic'}({self.key(None)} = {self.source})"
 
         def __eq__(self, other):
             return (
@@ -146,6 +151,7 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
         """
         is_partial = bool(parser.try_consume(IdentifierToken("PARTIAL")))
         is_macro = not is_partial and bool(parser.try_consume(IdentifierToken("MACRO")))
+
         return cls.consume_inner(parser, is_partial, is_macro, scope)
 
     @classmethod
@@ -168,18 +174,37 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
                 )
 
                 if isinstance(call_target, AbstractCallAssembly):
-                    raise RuntimeError
+                    # should not be reachable
+
+                    raise throw_positioned_error(
+                        scope,
+                        list(call_target.get_tokens()),
+                        "Must be not <call assembly> (internal error)",
+                        RuntimeError,
+                    )
 
             else:
                 name = [parser.consume(IdentifierToken, err_arg=scope)]
 
-                while parser.try_consume(SpecialToken(":")):
-                    name.append(parser.consume(IdentifierToken, err_arg=scope))
+                while expr := parser.try_consume(SpecialToken(":")):
+                    # todo: allow identifier-like
+                    part = parser.try_consume(IdentifierToken)
+
+                    if part is None:
+                        raise throw_positioned_error(
+                            scope,
+                            [expr, parser[0]],
+                            "<identifier> expected after '.'",
+                        )
+
+                    name.append(part)
 
                 call_target = MacroAccessExpression(name)
 
         if call_target is None:
-            raise throw_positioned_syntax_error(
+            # should be unreachable
+
+            raise throw_positioned_error(
                 scope,
                 parser.try_inspect(),
                 "expected <expression> to be called (did you forget the prefix?)"
@@ -189,17 +214,29 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
 
         args: typing.List[AbstractCallAssembly.IArg] = []
 
-        parser.consume(SpecialToken("("), err_arg=scope)
+        if not (opening_bracket := parser.try_consume(SpecialToken("("))):
+            raise throw_positioned_error(
+                scope,
+                parser[0],
+                "Expected '(' after <call target>",
+            )
 
         has_seen_keyword_arg = False
 
         while not (bracket := parser.try_consume(SpecialToken(")"))):
-            if (
-                isinstance(parser[0], IdentifierToken)
-                and parser[1] == SpecialToken("=")
-                and not is_macro
-            ):
-                key = parser.consume(IdentifierToken)
+            parser.save()
+
+            identifier = parser.try_parse_identifier_like()
+            is_keyword = identifier and parser.try_inspect() == SpecialToken("=") and not is_macro
+
+            if is_keyword:
+                parser.discard_save()
+            else:
+                parser.rollback()
+
+            if is_keyword:
+                key = identifier
+
                 parser.consume(SpecialToken("="))
 
                 is_dynamic = is_partial and bool(parser.try_consume(SpecialToken("?")))
@@ -230,16 +267,51 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
                     args.append(AbstractCallAssembly.StarArg(expr))
 
                 else:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope,
                         parser.try_inspect(),
                         "*<arg> only allowed before keyword arguments!",
                     )
 
             elif not has_seen_keyword_arg:
-                if is_macro and parser[0] == SpecialToken("{"):
+                if is_macro and ((inner_opening_bracket := parser[0]) == SpecialToken("[")):
+                    parser.consume(SpecialToken("["))
+
+                    to_be_stored_at = []
+
+                    while not parser.try_inspect() == SpecialToken("]"):
+                        parser.try_consume(SpecialToken("$"))
+                        base = parser.try_parse_identifier_like()
+
+                        if base is None:
+                            raise throw_positioned_error(
+                                scope,
+                                [inner_opening_bracket, parser[0]],
+                                "Expected <expression> after ','",
+                            )
+
+                        to_be_stored_at.append(base)
+
+                        if not parser.try_consume(SpecialToken(",")):
+                            break
+
+                    if not parser.try_consume(SpecialToken("]")):
+                        raise throw_positioned_error(
+                            scope,
+                            [inner_opening_bracket, parser[0]],
+                            "Expected ']' closing '['",
+                        )
+
                     expr = parser.parse_body(scope=scope)
+                    expr.to_be_stored_at = to_be_stored_at
                     is_dynamic = False
+
+                # todo: maybe parse here also partial variable names
+                elif is_macro and parser[0] == SpecialToken("{"):
+                    expr = parser.parse_body(scope=scope)
+                    expr.to_be_stored_at = []
+                    is_dynamic = False
+
                 else:
                     is_dynamic = is_partial and bool(
                         parser.try_consume(SpecialToken("?"))
@@ -251,14 +323,14 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
                         if parser[0] == SpecialToken(")"):
                             break
 
-                        raise throw_positioned_syntax_error(
+                        raise throw_positioned_error(
                             scope, parser[0], "<expression> expected"
                         )
 
                 args.append(AbstractCallAssembly.Arg(expr, is_dynamic))
 
             else:
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     parser.try_inspect(),
                     "pure <arg> only allowed before keyword arguments",
@@ -268,17 +340,29 @@ class AbstractCallAssembly(AbstractAssemblyInstruction, AbstractAccessExpression
                 break
 
         if bracket is None and not parser.try_consume(SpecialToken(")")):
-            raise throw_positioned_syntax_error(
-                scope, parser.try_inspect(), "expected ')'"
+            raise throw_positioned_error(
+                scope,
+                [opening_bracket, parser[0]],
+                "expected ')' matching '('",
             )
 
-        if allow_target and parser.try_consume_multi(
-            [
-                SpecialToken("-"),
-                SpecialToken(">"),
-            ]
-        ):
+        if allow_target and (arrow_0 := parser.try_consume(SpecialToken("-"))):
+
+            if not (arrow_1 := parser.try_consume(SpecialToken(">"))):
+                raise throw_positioned_error(
+                    scope,
+                    [arrow_0, parser[0]],
+                    "expected '>' after '-' to fill out <target> expression",
+                )
+
             target = parser.try_consume_access_to_value(scope=scope)
+
+            if target is None:
+                raise throw_positioned_error(
+                    scope,
+                    [arrow_0, arrow_1],
+                    "expected <target> expression after '->'",
+                )
         else:
             target = None
 
