@@ -7,9 +7,8 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.AbstractBase import StaticIdentifier
 from bytecodemanipulation.assembler.Lexer import SpecialToken
 from bytecodemanipulation.assembler.Parser import Parser
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
-from bytecodemanipulation.assembler.util.tokenizer import IdentifierToken
 from bytecodemanipulation.data.shared.instructions.AbstractInstruction import (
     AbstractAssemblyInstruction,
 )
@@ -30,6 +29,22 @@ class AbstractFunctionDefinitionAssembly(AbstractAssemblyInstruction, abc.ABC):
     def consume(
         cls, parser: "Parser", scope: ParsingScope
     ) -> "AbstractFunctionDefinitionAssembly":
+        prefix = ""
+
+        while parser.try_consume(SpecialToken("|")):
+            prefix += "|"
+
+        if prefix == "":
+            while parser.try_consume(SpecialToken(":")):
+                prefix += ":"
+
+        elif error := parser.try_consume(SpecialToken(":")):
+            raise throw_positioned_error(
+                scope,
+                error,
+                "Cannot parse '|' and ':' as inter-fix for <function name>",
+            )
+
         func_name = parser.parse_identifier_like(scope)
 
         bound_variables: typing.List[typing.Tuple[IIdentifierAccessor, bool]] = []
@@ -53,77 +68,115 @@ class AbstractFunctionDefinitionAssembly(AbstractAssemblyInstruction, abc.ABC):
 
             parser.consume(SpecialToken(">"), err_arg=scope)
 
-        parser.consume(SpecialToken("("), err_arg=scope)
+        if (opening_bracket := parser.try_consume(SpecialToken("("))) is None:
+            raise throw_positioned_error(
+                scope,
+                parser[0],
+                "expected '(' after <identifier>"
+            )
 
         while parser.try_inspect() != SpecialToken(")"):
             arg = None
 
             star = parser.try_consume(SpecialToken("*"))
             star_star = parser.try_consume(SpecialToken("*"))
-            identifier = parser.try_consume(IdentifierToken)
+            identifier = parser.try_parse_identifier_like()
 
             if not identifier:
                 if star:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope,
                         [star, star_star],
-                        "Expected <expression> after '*'"
+                        "Expected <identifier> after '*'"
                         if not star_star
-                        else "Expected <expression> after '**'",
+                        else "Expected <identifier> after '**'",
                     )
 
                 break
 
             if not star:
-                if parser.try_consume(SpecialToken("=")):
+                if equal_sign := parser.try_consume(SpecialToken("=")):
                     default_value = parser.try_parse_data_source(
                         allow_primitives=True, include_bracket=False, allow_op=True
                     )
 
                     if default_value is None:
-                        raise SyntaxError
+                        raise throw_positioned_error(
+                            scope,
+                            [identifier, equal_sign, parser[0]],
+                            "expected <expression>",
+                        )
 
-                    arg = AbstractCallAssembly.KwArg(identifier, default_value)
+                    arg = AbstractCallAssembly.KwArg(identifier, default_value, token=identifier)
 
             if not arg:
                 if star_star:
-                    arg = AbstractCallAssembly.KwArgStar(identifier)
+                    arg = AbstractCallAssembly.KwArgStar(identifier, token=identifier)
+
                 elif star:
-                    arg = AbstractCallAssembly.StarArg(identifier)
+                    arg = AbstractCallAssembly.StarArg(identifier, token=identifier)
+
                 else:
-                    arg = AbstractCallAssembly.Arg(identifier)
+                    arg = AbstractCallAssembly.Arg(identifier, token=identifier)
 
             args.append(arg)
 
             if not parser.try_consume(SpecialToken(",")):
                 break
 
-        parser.consume(SpecialToken(")"))
+        if not parser.try_consume(SpecialToken(")")):
+            raise throw_positioned_error(
+                scope,
+                [opening_bracket, parser[0]],
+                "expected ')' matching '(' in function definition",
+            )
 
         if expr := parser.try_consume(SpecialToken("<")):
-            raise throw_positioned_syntax_error(
+            if bound_variables:
+                raise throw_positioned_error(
+                    scope,
+                    expr,
+                    "got '<' after '(' ... ')' expression, where a '{' ... '}' (code block) was expected"
+                )
+
+            raise throw_positioned_error(
                 scope,
                 expr,
                 "Respect ordering (got 'args' before 'captured'): DEF ['name'] ['captured'] ('args') [-> 'target'] { code }",
             )
 
-        if parser.try_consume(SpecialToken("-")) and parser.try_consume(
+        if (arrow_0 := parser.try_consume(SpecialToken("-"))) and (arrow_1 := parser.try_consume(
             SpecialToken(">")
-        ):
+        )):
             target = parser.try_consume_access_to_value(scope=scope)
+
+            if target is None:
+                raise throw_positioned_error(
+                    scope,
+                    [arrow_0, arrow_1, parser[0]],
+                    "expected <expression> after '->' as target"
+                )
+
+        elif arrow_0:
+            raise throw_positioned_error(
+                scope,
+                [arrow_0, parser[0]],
+                "expected '>' after '-' to complete <target> expression",
+            )
+
         else:
             target = None
 
         body = parser.parse_body(scope=scope)
 
         if expr := parser.try_consume(SpecialToken("-")):
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope,
                 expr,
                 "Respect ordering (got 'code' before 'target'): DEF ['name'] ['captured'] ('args') [-> 'target'] { code }",
             )
 
-        return cls(func_name, bound_variables, args, body, target)
+        return cls(func_name, bound_variables, args, body, target, prefix=prefix)
 
     def __init__(
         self,
@@ -132,6 +185,7 @@ class AbstractFunctionDefinitionAssembly(AbstractAssemblyInstruction, abc.ABC):
         args: typing.List[AbstractCallAssembly.IArg],
         body: CompoundExpression,
         target: AbstractAccessExpression | None = None,
+        prefix="",
     ):
         self.func_name = (
             func_name if not isinstance(func_name, str) else StaticIdentifier(func_name)
@@ -163,6 +217,7 @@ class AbstractFunctionDefinitionAssembly(AbstractAssemblyInstruction, abc.ABC):
         self.args = args
         self.body = body
         self.target = target
+        self.prefix = prefix
 
     def visit_parts(
         self,
@@ -192,7 +247,7 @@ class AbstractFunctionDefinitionAssembly(AbstractAssemblyInstruction, abc.ABC):
         )
 
     def __repr__(self):
-        return f"DEF({self.func_name}<{repr([(name[0](None), name[1]) for name in self.bound_variables])[1:-1]}>({repr(self.args)[1:-1]}){'-> ' + repr(self.target) if self.target else ''} {{ {self.body} }})"
+        return f"DEF({self.prefix}{self.func_name}<{repr([('!:' if name[1] else '') + name[0](None) for name in self.bound_variables])[1:-1]}>({repr(self.args)[1:-1]}){'-> ' + repr(self.target) if self.target else ''} {{ {self.body} }})"
 
     def copy(self) -> "AbstractFunctionDefinitionAssembly":
         return type(self)(
@@ -200,5 +255,6 @@ class AbstractFunctionDefinitionAssembly(AbstractAssemblyInstruction, abc.ABC):
             self.bound_variables.copy(),
             [arg.copy() for arg in self.args],
             self.body.copy(),
-            self.target.copy() if self.target else None,
+            target=self.target.copy() if self.target else None,
+            prefix=self.prefix,
         )
