@@ -8,7 +8,7 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.AbstractBase import StaticIdentifier
 from bytecodemanipulation.assembler.Lexer import SpecialToken
 from bytecodemanipulation.assembler.Parser import Parser
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
 from bytecodemanipulation.data.shared.expressions.CompoundExpression import (
     CompoundExpression,
@@ -29,16 +29,18 @@ class AbstractIFAssembly(AbstractAssemblyInstruction, abc.ABC):
         )
 
         if source is None:
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope, parser.try_inspect(), "expected <expression>"
             )
 
         if parser.try_consume(SpecialToken("'")):
-            label_name = parser.parse_identifier_like(scope)
+            label_name = parser.parse_jump_target(scope)
+
             if not parser.try_consume(SpecialToken("'")):
-                raise throw_positioned_syntax_error(
-                    scope, parser.try_inspect(), "expected '"
+                raise throw_positioned_error(
+                    scope, parser.try_inspect(), "expected ' after label name declaration"
                 )
+
         else:
             label_name = None
 
@@ -54,18 +56,18 @@ class AbstractIFAssembly(AbstractAssemblyInstruction, abc.ABC):
         self,
         source: AbstractSourceExpression,
         body: CompoundExpression,
-        label_name: IIdentifierAccessor | str = None,
+        label_name: typing.List[IIdentifierAccessor] | str = None,
     ):
         self.source = source
         self.body = body
         self.label_name = (
             label_name
             if not isinstance(label_name, str)
-            else StaticIdentifier(label_name)
-        )
+            else [StaticIdentifier(e) for e in label_name.split(":")]
+        ) if label_name is not None else None
 
     def copy(self):
-        return type(self)(self.source.copy(), self.body.copy(), self.label_name)
+        return type(self)(self.source.copy(), self.body.copy(), self.label_name.copy())
 
     def __eq__(self, other):
         return (
@@ -76,8 +78,8 @@ class AbstractIFAssembly(AbstractAssemblyInstruction, abc.ABC):
         )
 
     def __repr__(self):
-        c = "'"
-        return f"IF({self.source}{'' if self.label_name is None else ', label='+c+repr(self.label_name)+c}) -> {{{self.body}}}"
+        c = '"'
+        return f"IF({self.source}{'' if self.label_name is None else ', label='+c+':'.join(map(repr, self.label_name))+c}) -> {{{self.body}}}"
 
     def visit_parts(
         self,
@@ -102,12 +104,14 @@ class AbstractIFAssembly(AbstractAssemblyInstruction, abc.ABC):
         return visitor(self, (self.body.visit_assembly_instructions(visitor),))
 
     def get_labels(self, scope: ParsingScope) -> typing.Set[str]:
+        label_name = ":".join(e(scope) for e in self.label_name) if self.label_name is not None else None
+
         return (
             set()
             if self.label_name is None
             else {
-                self.label_name(scope),
-                self.label_name(scope) + "_END",
-                self.label_name(scope) + "_HEAD",
+                label_name,
+                label_name + ":END",
+                label_name + ":HEAD",
             }
         ) | self.body.get_labels(scope)
