@@ -1,11 +1,17 @@
+import traceback
 import typing
 from abc import ABC
 
+from bytecodemanipulation.assembler.util.tokenizer import IntegerToken
+from bytecodemanipulation.MutableFunctionHelpers import capture_local
+
+from bytecodemanipulation.MutableFunctionHelpers import outer_return
+
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.assembler.AbstractBase import JumpToLabel
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.Lexer import SpecialToken
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.assembler.util.tokenizer import IdentifierToken
 from bytecodemanipulation.data.shared.instructions.AbstractInstruction import (
     AbstractAssemblyInstruction,
@@ -22,9 +28,9 @@ from bytecodemanipulation.data.shared.expressions.GlobalAccessExpression import
 from bytecodemanipulation.data.shared.expressions.LocalAccessExpression import (
     LocalAccessExpression,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 if typing.TYPE_CHECKING:
@@ -37,6 +43,20 @@ LOCAL_TO_DEREF_OPCODES = {
 }
 
 
+def _manipulate_stack_trace(exc: Exception, file: str, func_name: str, line: int, column: int, span: int):
+    trace = traceback.extract_stack()
+
+    entry = traceback.FrameSummary(
+        filename=file,
+        lineno=line,
+        name=func_name,
+    )
+
+    trace.append(entry)
+
+    # exc.__traceback__ = traceback.TracebackException(*trace).__traceback__
+
+
 class MacroAssembly(AbstractAssemblyInstruction):
     # 'MACRO' ['ASSEMBLY'] [{<namespace> ':'}] <name> ['(' <param> \[{',' <param>}] ')'] ['->' <data type>] '{' <assembly code> '}', where param is ['!'] \<name> [<data type>]
     NAME = "MACRO"
@@ -86,6 +106,9 @@ class MacroAssembly(AbstractAssemblyInstruction):
     class CodeBlockDataType(AbstractDataType):
         IDENTIFIER = "CODE_BLOCK"
 
+        def __init__(self, count=0):
+            self.count = count
+
         def is_match(
             self,
             arg: "MacroAssembly.MacroArg",
@@ -142,7 +165,7 @@ class MacroAssembly(AbstractAssemblyInstruction):
             for arg in args:
                 bytecode += arg.emit_bytecodes(function, scope)
 
-            bytecode += [Instruction(function, -1, "BUILD_LIST", arg=len(args))]
+            bytecode += [Instruction("BUILD_LIST", arg=len(args))]
             return bytecode
 
     class MacroArg:
@@ -266,7 +289,7 @@ class MacroAssembly(AbstractAssemblyInstruction):
                     args[:] = prefix + [inner] + postfix
                     return macro, args
 
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope,
                 self.name_token,
                 f"Could not find overloaded variant of {':'.join(self.name)} with args {args}!",
@@ -278,18 +301,35 @@ class MacroAssembly(AbstractAssemblyInstruction):
             self.assemblies.append(macro)
 
     @classmethod
-    def _try_parse_arg_data_type(cls, parser: "Parser") -> AbstractDataType | None:
+    def _try_parse_arg_data_type(cls, parser: "Parser", scope: ParsingScope) -> AbstractDataType | None:
         if identifier := parser.try_inspect(IdentifierToken):
             if identifier.text == "CODE_BLOCK":
                 parser.consume(identifier)
-                inst = cls.CodeBlockDataType()
+
+                count = 0
+                if opening_bracket := parser.try_consume(SpecialToken("[")):
+                    if not (expr := parser.try_consume(IntegerToken)) or not expr.text.isdigit() or (count := int(expr.text)) < 0:
+                        raise throw_positioned_error(
+                            scope,
+                            [opening_bracket, parser[0]],
+                            "expected <integer> after '[' to declare count",
+                        )
+
+                    if not parser.try_consume(SpecialToken("]")):
+                        raise throw_positioned_error(
+                            scope,
+                            [opening_bracket, parser[0]],
+                            "expected ']' closing '[' in CODE_BLOCK",
+                        )
+
+                inst = cls.CodeBlockDataType(count=count)
                 return inst
 
             elif identifier.text == "VARIABLE_ARG":
                 parser.consume(identifier)
                 # todo: add check that it is the only * arg
                 if parser.try_consume(SpecialToken("[")):
-                    inner_type = cls._try_parse_arg_data_type(parser)
+                    inner_type = cls._try_parse_arg_data_type(parser, scope)
                     if inner_type is None:
                         return
 
@@ -331,7 +371,7 @@ class MacroAssembly(AbstractAssemblyInstruction):
                 i += 1
                 args.append(arg)
 
-                data_type = cls._try_parse_arg_data_type(parser)
+                data_type = cls._try_parse_arg_data_type(parser, scope)
 
                 parser.save()
                 if data_type is not None:
@@ -341,13 +381,13 @@ class MacroAssembly(AbstractAssemblyInstruction):
                     parser.rollback()
 
                 if not parser.try_consume(SpecialToken(",")):
-                    parser.consume(SpecialToken(")"))
+                    parser.consume(SpecialToken(")"), err_arg=scope)
                     break
 
         if parser.try_consume(SpecialToken("-")):
             parser.consume(SpecialToken(">"), err_arg=scope)
 
-            return_type = cls._try_parse_arg_data_type(parser)
+            return_type = cls._try_parse_arg_data_type(parser, scope)
         else:
             return_type = None
 
@@ -420,10 +460,20 @@ class MacroAssembly(AbstractAssemblyInstruction):
         scope = scope.copy()
         scope.scope_path = self.scope_path
         scope.module_file = self.module_path
+        scope.current_macro_assembly = self
 
         bytecode = []
 
         for arg_decl, arg_code in zip(self.args, args):
+            if isinstance(arg_decl.data_type_annotation, MacroAssembly.CodeBlockDataType):
+                if isinstance(arg_code, CompoundExpression) and hasattr(arg_code, "to_be_stored_at"):
+                    if len(arg_code.to_be_stored_at) != arg_decl.data_type_annotation.count:
+                        raise throw_positioned_error(
+                            scope,
+                            arg_decl.name,
+                            f"Expected {arg_decl.data_type_annotation.count} dynamic name entries, got {len(arg_code.to_be_stored_at)}"
+                        )
+
             if arg_decl.is_static:
                 scope.macro_parameter_namespace[arg_decl.name.text] = arg_decl.name.text
             else:
@@ -443,25 +493,21 @@ class MacroAssembly(AbstractAssemblyInstruction):
                     )
                     bytecode.append(
                         Instruction(
-                            function,
-                            -1,
                             Opcodes.STORE_FAST,
                             var_name,
-                            _decode_next=False,
                         )
                     )
+
                 else:
                     arg_names.append(var_name := scope.scope_name_generator(f"arg_{i}"))
                     bytecode += arg_code.emit_bytecodes(function, scope)
                     bytecode.append(
                         Instruction(
-                            function,
-                            -1,
                             Opcodes.STORE_FAST,
                             var_name,
-                            _decode_next=False,
                         )
                     )
+
             else:
                 arg_names.append(None)
 
@@ -481,7 +527,7 @@ class MacroAssembly(AbstractAssemblyInstruction):
                 Opcodes.MACRO_PARAMETER_EXPANSION,
             ):
                 if instr.arg_value not in arg_decl_lookup:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope,
                         self.name,
                         f"macro name {instr.arg_value} not found in scope",
@@ -519,7 +565,7 @@ class MacroAssembly(AbstractAssemblyInstruction):
 
             elif instr.opcode == Opcodes.MACRO_STORE_PARAMETER:
                 if instr.arg_value not in arg_decl_lookup:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope,
                         self.name,
                         f"macro name {instr.arg_value} not found in scope",
@@ -541,18 +587,21 @@ class MacroAssembly(AbstractAssemblyInstruction):
                             bytecode += instructions
                             continue
 
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope,
                         self.name,
                         f"Tried to override non-static MACRO parameter '{instr.arg_value}'; This is not allowed as the parameter will be evaluated on each access!",
                         RuntimeError,
                     )
 
-            elif instr.has_local() and instr.arg_value.startswith("MACRO_"):
+            elif instr.has_local() and instr.arg_value.startswith(":"):
                 instr.change_arg_value(
-                    local_prefix + ":" + instr.arg_value.removeprefix("MACRO_")
+                    instr.arg_value.removeprefix(":")
                 )
 
+            elif instr.has_local():
+                instr.change_arg_value(local_prefix + ":" + instr.arg_value)
+
             elif instr.opcode == Opcodes.MACRO_RETURN_VALUE:
                 if self.return_type is None:
                     raise SyntaxError(
@@ -562,9 +611,28 @@ class MacroAssembly(AbstractAssemblyInstruction):
                 instr.change_opcode(Opcodes.JUMP_ABSOLUTE, JumpToLabel(end_target))
 
         if requires_none_load:
-            bytecode.append(Instruction(function, -1, Opcodes.LOAD_CONST, None))
+            bytecode.append(Instruction(Opcodes.LOAD_CONST, None))
 
-        bytecode.append(Instruction(function, -1, Opcodes.BYTECODE_LABEL, end_target))
+        bytecode.append(Instruction(Opcodes.BYTECODE_LABEL, end_target))
+
+        # handle = Instruction(Opcodes.NOP)
+        # handle.insert_after(
+        #     [
+        #         Instruction(Opcodes.POP_TOP),
+        #         Instruction(Opcodes.ROT_TWO),  # TOS would be the exception
+        #         Instruction(Opcodes.POP_TOP),
+        #         Instruction(Opcodes.LOAD_CONST, _manipulate_stack_trace),
+        #         Instruction(Opcodes.ROT_TWO),
+        #         Instruction(Opcodes.LOAD_CONST, scope.module_file),
+        #         Instruction(Opcodes.LOAD_CONST, "".join(map(lambda e: e.text, self.name))),
+        #         Instruction(Opcodes.LOAD_CONST, 0),  # todo: load position info (line)
+        #         Instruction(Opcodes.LOAD_CONST, 0),  # todo: load position info (column)
+        #         Instruction(Opcodes.LOAD_CONST, 0),  # todo: load position info (span)
+        #         Instruction(Opcodes.CALL_FUNCTION, arg=6),
+        #         Instruction(Opcodes.RAISE_VARARGS, arg=0),
+        #     ]
+        # )
+        # function.exception_table.add_handle(handle, inner_bytecode)
 
         return bytecode
 
@@ -598,24 +666,61 @@ class MacroAssembly(AbstractAssemblyInstruction):
             self, function: MutableFunction, scope: ParsingScope
         ) -> typing.List[Instruction]:
             macro_exit_label = scope.scope_name_generator("macro_exit")
-            return [
+
+            builder = self.function.create_filled_builder()
+
+            instructions = [
                 (
-                    instr.copy(owner=function)
+                    instr
                     if instr.opcode != Opcodes.RETURN_VALUE
-                    else instr.copy(owner=function)
+                    else instr
                     .change_opcode(Opcodes.POP_TOP)
                     .insert_after(
                         Instruction(
-                            function,
-                            -1,
                             Opcodes.JUMP_ABSOLUTE,
                             JumpToLabel(macro_exit_label),
                         )
                     )
                 )
                 if not instr.has_local() or instr.arg_value not in self.scoped_names
-                else instr.copy(owner=function).change_opcode(
+                else instr.copy().change_opcode(
                     LOCAL_TO_DEREF_OPCODES[instr.opcode]
                 )
-                for instr in self.function.instructions
-            ] + [Instruction(function, -1, Opcodes.BYTECODE_LABEL, macro_exit_label)]
+                for instr in builder.temporary_instructions
+            ]
+
+            captured_locals = set()
+            for instr in instructions:
+                if instr.opcode == Opcodes.LOAD_GLOBAL:
+                    target = self.function.target.__globals__.get(instr.arg_value, None)
+
+                    if target == capture_local:
+                        call = next(instr.trace_stack_position_use(0))
+
+                        if call.opcode != Opcodes.CALL_FUNCTION:
+                            raise ValueError(call)
+
+                        arg = next(call.trace_stack_position(0))
+
+                        if arg.opcode != Opcodes.LOAD_CONST:
+                            raise ValueError(arg)
+
+                        captured_locals.add(arg.arg_value)
+                        arg.change_opcode(Opcodes.NOP)
+                        call.change_opcode(Opcodes.LOAD_CONST, None)
+                        instr.change_opcode(Opcodes.NOP)
+
+            for instr in instructions:
+                if instr.opcode == Opcodes.LOAD_GLOBAL:
+                    target = self.function.target.__globals__.get(instr.arg_value, None)
+
+                    if target == outer_return:
+                        call = next(instr.trace_stack_position_use(0))
+
+                        instr.change_opcode(Opcodes.NOP)
+                        call.change_opcode(Opcodes.RETURN_VALUE)
+
+                elif instr.has_local() and instr.arg_value in captured_locals:
+                    instr.change_arg_value(":" + instr.arg_value)
+
+            return instructions + [Instruction(Opcodes.BYTECODE_LABEL, macro_exit_label)]
