@@ -8,9 +8,9 @@ from bytecodemanipulation.assembler.AbstractBase import IAssemblyStructureVisita
 from bytecodemanipulation.assembler.Parser import Parser
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 @Parser.register
@@ -56,4 +56,4 @@ class MacroReturnAssembly(AbstractAssemblyInstruction):
     ) -> typing.List[Instruction]:
         expr_bytecode = self.expr.emit_bytecodes(function, scope) if self.expr else []
 
-        return expr_bytecode + [Instruction(function, -1, Opcodes.MACRO_RETURN_VALUE)]
+        return expr_bytecode + [Instruction(Opcodes.MACRO_RETURN_VALUE)]
