@@ -4,16 +4,18 @@ import typing
 from collections import namedtuple
 
 from bytecodemanipulation.assembler.AbstractBase import AbstractSourceExpression
+from bytecodemanipulation.assembler.target import assembly
 from bytecodemanipulation.assembler.util.tokenizer import AbstractToken
 from bytecodemanipulation.assembler.Lexer import SpecialToken
 from bytecodemanipulation.assembler.AbstractBase import IAssemblyStructureVisitable
-from bytecodemanipulation.assembler.syntax_errors import throw_positioned_syntax_error
+from bytecodemanipulation.assembler.syntax_errors import throw_positioned_error
 from bytecodemanipulation.assembler.util.parser import AbstractExpression
 from bytecodemanipulation.assembler.AbstractBase import AbstractAccessExpression
 from bytecodemanipulation.data.shared.instructions.AbstractInstruction import (
     AbstractAssemblyInstruction,
 )
-from bytecodemanipulation.MutableFunction import MutableFunction, Instruction
+from bytecodemanipulation.MutableFunction import MutableFunction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 
 
@@ -30,6 +32,9 @@ class AbstractOperator(abc.ABC):
     ) -> typing.List[Instruction]:
         raise NotImplementedError
 
+    def evaluate_static_value(self, scope: ParsingScope, *parameters: AbstractSourceExpression):
+        raise NotImplementedError
+
 
 OperatorArgValue = namedtuple("OperatorArgValue", "index")
 
@@ -48,8 +53,10 @@ class OpcodeBaseOperator(AbstractOperator):
             ],
         ]
         | OperatorArgValue,
+        static_eval: typing.Callable[[ParsingScope, typing.List[AbstractSourceExpression]], typing.Any] = None
     ):
         self.opcodes = opcodes
+        self.static_eval = static_eval
 
     def emit_bytecodes(
         self,
@@ -64,7 +71,7 @@ class OpcodeBaseOperator(AbstractOperator):
 
         for opcode in self.opcodes:
             if isinstance(opcode, (int, str)):
-                bytecode.append(Instruction(function, -1, opcode))
+                bytecode.append(Instruction(opcode))
             elif isinstance(opcode, tuple):
                 if isinstance(opcode[1], typing.Callable):
                     bytecode.append(
@@ -76,7 +83,7 @@ class OpcodeBaseOperator(AbstractOperator):
                         )
                     )
                 else:
-                    bytecode.append(Instruction(function, -1, opcode[0], arg=opcode[1]))
+                    bytecode.append(Instruction(opcode[0], arg=opcode[1]))
             elif isinstance(opcode, OperatorArgValue):
                 bytecode += parameters[opcode.index].emit_bytecodes(function, scope)
             elif hasattr(opcode, "emit_bytecodes"):
@@ -86,6 +93,12 @@ class OpcodeBaseOperator(AbstractOperator):
 
         return bytecode
 
+    def evaluate_static_value(self, scope: ParsingScope, *parameters: AbstractSourceExpression):
+        if not self.static_eval:
+            raise NotImplementedError
+
+        return self.static_eval(scope, list(parameters))
+
 
 class AbstractOpAssembly(
     AbstractAssemblyInstruction, AbstractAccessExpression, abc.ABC
@@ -138,6 +151,9 @@ class AbstractOpAssembly(
         def __repr__(self):
             raise NotImplementedError
 
+        def evaluate_static_value(self, scope: ParsingScope):
+            raise NotImplementedError
+
     class SingleOperation(IOperation):
         def __init__(
             self,
@@ -174,12 +190,15 @@ class AbstractOpAssembly(
                     function, scope, self.expression
                 )
             except:
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     self.operator_token,
                     "during emitting bytecode of singleton operator",
                 )
 
+        def evaluate_static_value(self, scope: ParsingScope):
+            return self.base.SINGLE_OPS[self.operator].evaluate_static_value(scope, self.expression)
+
         def visit_parts(
             self,
             visitor: typing.Callable[[IAssemblyStructureVisitable, tuple], typing.Any],
@@ -242,12 +261,15 @@ class AbstractOpAssembly(
                     function, scope, self.lhs, self.rhs
                 )
             except:
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     self.operator_token,
                     "during emitting bytecode of binary operation",
                 )
 
+        def evaluate_static_value(self, scope: ParsingScope):
+            return self.base.BINARY_OPS[self.operator].evaluate_static_value(scope, self.lhs, self.rhs)
+
         def visit_parts(
             self,
             visitor: typing.Callable[[IAssemblyStructureVisitable, tuple], typing.Any],
@@ -303,13 +325,18 @@ class AbstractOpAssembly(
                 return self.base.PREFIX_OPERATORS[self.operator][0].emit_bytecodes(
                     function, scope, *self.args
                 )
+            except SyntaxError:
+                raise
             except:
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     self.operator_token,
                     "during emitting bytecode of binary operation",
                 )
 
+        def evaluate_static_value(self, scope: ParsingScope):
+            return self.base.PREFIX_OPERATORS[self.operator][0].evaluate_static_value(scope, *self.args)
+
         def visit_parts(
             self,
             visitor: typing.Callable[[IAssemblyStructureVisitable, tuple], typing.Any],
@@ -342,7 +369,7 @@ class AbstractOpAssembly(
         if expr := cls.try_consume_prefix(parser, scope):
             return cls(expr, cls.try_consume_arrow(parser, scope))
 
-        raise throw_positioned_syntax_error(
+        raise throw_positioned_error(
             scope,
             parser.try_inspect(),
             "expected <operator> or <expression> <operator> ...",
@@ -367,7 +394,7 @@ class AbstractOpAssembly(
     ) -> AbstractAccessExpression | None:
         if parser.try_consume(SpecialToken("-")):
             if not parser.try_consume(SpecialToken(">")):
-                raise throw_positioned_syntax_error(
+                raise throw_positioned_error(
                     scope,
                     parser[-1:1] + [scope.last_base_token],
                     "expected '>' after '-' to complete '->'",
@@ -449,9 +476,9 @@ class AbstractOpAssembly(
         bracket = parser.try_consume(SpecialToken("("))
 
         if bracket is None and has_brackets is True:
-            raise throw_positioned_syntax_error(scope, parser[0], "expected '('")
+            raise throw_positioned_error(scope, parser[0], "expected '('")
         elif bracket is not None and has_brackets is False:
-            raise throw_positioned_syntax_error(
+            raise throw_positioned_error(
                 scope,
                 operator_tokens + [parser[0]],
                 "did not expect '(' after operator name",
@@ -471,7 +498,7 @@ class AbstractOpAssembly(
                 )
 
                 if expr is None:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope, parser[0], "expected <expression>"
                     )
 
@@ -480,12 +507,12 @@ class AbstractOpAssembly(
                 coma = parser.try_consume(SpecialToken(","))
 
                 if coma is None and has_coma_sep is True:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope, parser[0], "expected ','"
                     )
 
                 elif coma is not None and has_coma_sep is False:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope, coma, "did not expect ',' for separation of args"
                     )
         else:
@@ -500,7 +527,7 @@ class AbstractOpAssembly(
                 )
 
                 if expr is None:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope, parser[0], "expected <expression>"
                     )
 
@@ -513,17 +540,17 @@ class AbstractOpAssembly(
                     and has_coma_sep is True
                     and parser[0] != SpecialToken(")")
                 ):
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope, parser[0], "expected ','"
                     )
 
                 elif coma is not None and has_coma_sep is False:
-                    raise throw_positioned_syntax_error(
+                    raise throw_positioned_error(
                         scope, coma, "did not expect ',' for separation of args"
                     )
 
         if bracket and not parser.try_consume(SpecialToken(")")):
-            raise throw_positioned_syntax_error(scope, parser[0], "expected ')'")
+            raise throw_positioned_error(scope, parser[0], "expected ')'")
 
         return cls.PrefixOperator(operator, operator_tokens, args, base=cls)
 
@@ -565,6 +592,9 @@ class AbstractOpAssembly(
     ) -> typing.List[Instruction]:
         raise RuntimeError(f"cannot assign to an '{self.operation}' operator!")
 
+    def evaluate_static_value(self, scope: ParsingScope) -> typing.Any:
+        return self.operation.evaluate_static_value(scope)
+
     def visit_parts(
         self,
         visitor: typing.Callable[
