@@ -5,9 +5,9 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.data.shared.instructions.ClassDefinitionAssembly import (
     AbstractClassDefinitionAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 @Parser.register
@@ -15,15 +15,15 @@ class ClassDefinitionAssembly(AbstractClassDefinitionAssembly):
     def emit_bytecodes(
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
-        inner_scope = scope.copy(sub_scope_name=self.name(scope))
+        inner_scope = scope.copy(sub_scope_name=self.name(scope), shared_locals=False)
 
         target = MutableFunction(lambda: None)
 
         inner_bytecode = [
-            Instruction(target, -1, Opcodes.LOAD_NAME, "__name__"),
-            Instruction(target, -1, Opcodes.STORE_NAME, "__module__"),
-            Instruction(target, -1, Opcodes.LOAD_CONST, self.name(scope)),
-            Instruction(target, -1, Opcodes.STORE_NAME, "__qualname__"),
+            Instruction(Opcodes.LOAD_NAME, "__name__"),
+            Instruction(Opcodes.STORE_NAME, "__module__"),
+            Instruction(Opcodes.LOAD_CONST, self.name(scope)),
+            Instruction(Opcodes.STORE_NAME, "__qualname__"),
         ]
 
         raw_inner_code = self.code_block.emit_bytecodes(target, inner_scope)
@@ -39,22 +39,22 @@ class ClassDefinitionAssembly(AbstractClassDefinitionAssembly):
         inner_bytecode += raw_inner_code
 
         if inner_bytecode:
-            inner_bytecode[-1].next_instruction = target.instructions[0]
+            inner_bytecode[-1].next_instruction = target.instruction_entry_point
 
         for i, instr in enumerate(inner_bytecode[:-1]):
             instr.next_instruction = inner_bytecode[i + 1]
 
-        target.assemble_instructions_from_tree(inner_bytecode[0])
+        target.instruction_entry_point = inner_bytecode[0]
         target.reassign_to_function()
 
         code_obj = target.target.__code__
 
         bytecode = [
-            Instruction(function, -1, Opcodes.LOAD_BUILD_CLASS),
-            Instruction(function, -1, Opcodes.LOAD_CONST, code_obj),
-            Instruction(function, -1, Opcodes.LOAD_CONST, self.name(scope)),
-            Instruction(function, -1, Opcodes.MAKE_FUNCTION, arg=0),
-            Instruction(function, -1, Opcodes.LOAD_CONST, self.name(scope)),
+            Instruction(Opcodes.LOAD_BUILD_CLASS),
+            Instruction(Opcodes.LOAD_CONST, code_obj),
+            Instruction(Opcodes.LOAD_CONST, self.name(scope)),
+            Instruction(Opcodes.MAKE_FUNCTION, arg=0),
+            Instruction(Opcodes.LOAD_CONST, self.name(scope)),
         ]
 
         for parent in self.parents:
@@ -64,7 +64,7 @@ class ClassDefinitionAssembly(AbstractClassDefinitionAssembly):
             )
 
         bytecode += [
-            Instruction(function, -1, Opcodes.CALL_FUNCTION, arg=2 + len(self.parents)),
-            Instruction(function, -1, Opcodes.STORE_FAST, self.name(scope)),
+            Instruction(Opcodes.CALL_FUNCTION, arg=2 + len(self.parents)),
+            Instruction(Opcodes.STORE_FAST, self.name(scope)),
         ]
         return bytecode
