@@ -5,7 +5,7 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.data.shared.instructions.LoadGlobalAssembly import (
     AbstractLoadGlobalAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 
@@ -19,6 +19,6 @@ class LoadGlobalAssembly(AbstractLoadGlobalAssembly):
         if value.isdigit():
             value = int(value)
 
-        return [Instruction(function, -1, "LOAD_GLOBAL", value)] + (
+        return [Instruction("LOAD_GLOBAL", value)] + (
             self.target.emit_bytecodes(function, scope) if self.target else []
         )
