@@ -3,7 +3,7 @@ import typing
 from bytecodemanipulation.data.shared.instructions.FunctionDefinitionAssembly import (
     AbstractFunctionDefinitionAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.assembler.AbstractBase import JumpToLabel
 from bytecodemanipulation.assembler.Parser import Parser
 from bytecodemanipulation.assembler.AbstractBase import ParsingScope
@@ -11,7 +11,7 @@ from bytecodemanipulation.data.shared.instructions.CallAssembly import (
     AbstractCallAssembly,
 )
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 from bytecodemanipulation.util import LambdaInstructionWalker
 
 
@@ -46,10 +46,10 @@ class FunctionDefinitionAssembly(AbstractFunctionDefinitionAssembly):
 
         if self.bound_variables:
             for name, is_static in self.bound_variables:
-                # print(name, name(scope), is_static)
+                print(name, name(scope), is_static)
                 inner_bytecode += [
-                    Instruction(target, -1, Opcodes.LOAD_DEREF, name(scope) + "%inner"),
-                    Instruction(target, -1, Opcodes.STORE_DEREF, name(scope)),
+                    Instruction(Opcodes.LOAD_DEREF, name(scope) + "%inner"),
+                    Instruction(Opcodes.STORE_DEREF, name(scope)),
                 ]
 
         inner_bytecode += self.body.emit_bytecodes(target, inner_scope)
@@ -97,14 +97,14 @@ class FunctionDefinitionAssembly(AbstractFunctionDefinitionAssembly):
 
             for name, is_static in self.bound_variables:
                 bytecode += [
-                    Instruction(function, -1, Opcodes.LOAD_FAST, name(scope)),
+                    Instruction(Opcodes.LOAD_FAST, name(scope)),
                     Instruction(
                         function, -1, Opcodes.STORE_DEREF, name(scope) + "%inner"
                     ),
                 ]
 
             bytecode += [
-                Instruction(function, -1, Opcodes.LOAD_CLOSURE, name(scope) + "%inner")
+                Instruction(Opcodes.LOAD_CLOSURE, name(scope) + "%inner")
                 for name, is_static in self.bound_variables
             ]
             bytecode.append(
@@ -114,24 +114,28 @@ class FunctionDefinitionAssembly(AbstractFunctionDefinitionAssembly):
             )
 
         target.argument_count = len(self.args)
+
+        if self.bound_variables:
+            target.argument_count += 1
+
         code_object = target.create_code_obj()
 
         bytecode += [
-            Instruction(function, -1, "LOAD_CONST", code_object),
+            Instruction("LOAD_CONST", code_object),
             Instruction(
                 function,
                 -1,
                 "LOAD_CONST",
                 self.func_name(scope) if self.func_name else "<lambda>",
             ),
-            Instruction(function, -1, "MAKE_FUNCTION", arg=flags),
+            Instruction("MAKE_FUNCTION", arg=flags),
         ]
 
         if self.target:
             bytecode += self.target.emit_store_bytecodes(function, scope)
         else:
             bytecode += [
-                Instruction(function, -1, Opcodes.STORE_FAST, self.func_name(scope)),
+                Instruction(Opcodes.STORE_FAST, self.func_name(scope)),
             ]
 
         return bytecode
