@@ -6,9 +6,9 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.data.shared.instructions.JumpAssembly import (
     AbstractJumpAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 @Parser.register
@@ -26,8 +26,6 @@ class JumpAssembly(AbstractJumpAssembly):
         if self.condition is None:
             return [
                 Instruction(
-                    function,
-                    -1,
                     Opcodes.JUMP_ABSOLUTE,
                     JumpToLabel(self.label_name_token.text),
                 )
@@ -35,8 +33,6 @@ class JumpAssembly(AbstractJumpAssembly):
 
         return self.condition.emit_bytecodes(function, scope) + [
             Instruction(
-                function,
-                -1,
                 Opcodes.POP_JUMP_IF_TRUE,
                 JumpToLabel(self.label_name_token.text),
             )
