@@ -5,7 +5,7 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.data.shared.instructions.PopElementAssembly import (
     AbstractPopElementAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 
@@ -15,6 +15,6 @@ class PopElementAssembly(AbstractPopElementAssembly):
         self, function: MutableFunction, scope: ParsingScope
     ) -> typing.List[Instruction]:
         return [
-            Instruction(function, -1, "POP_TOP", int(self.count.text))
+            Instruction("POP_TOP", int(self.count.text))
             for _ in range(int(self.count.text))
         ]
