@@ -5,9 +5,9 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.data.shared.instructions.RaiseAssembly import (
     AbstractRaiseAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 
 
 @Parser.register
@@ -17,4 +17,4 @@ class RaiseAssembly(AbstractRaiseAssembly):
     ) -> typing.List[Instruction]:
         return (
             [] if self.source is None else self.source.emit_bytecodes(function, scope)
-        ) + [Instruction(function, -1, Opcodes.RAISE_VARARGS, arg=1)]
+        ) + [Instruction(Opcodes.RAISE_VARARGS, arg=1)]
