@@ -5,7 +5,7 @@ from bytecodemanipulation.assembler.AbstractBase import ParsingScope
 from bytecodemanipulation.data.shared.instructions.StoreGlobalAssembly import (
     AbstractStoreGlobalAssembly,
 )
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 
@@ -21,4 +21,4 @@ class StoreGlobalAssembly(AbstractStoreGlobalAssembly):
 
         return (
             [] if self.source is None else self.source.emit_bytecodes(function, scope)
-        ) + [Instruction(function, -1, "STORE_GLOBAL", value)]
+        ) + [Instruction("STORE_GLOBAL", value)]
