@@ -3,8 +3,8 @@ import json
 import logging
 import os
 import sys
-from bytecodemanipulation.Opcodes import Opcodes, init_maps, OPNAME2CODE
-from bytecodemanipulation import Opcodes as OpcodesM
+from bytecodemanipulation.opcodes.Opcodes import Opcodes, init_maps, OPNAME2CODE
+from bytecodemanipulation.opcodes import Opcodes as OpcodesM
 from bytecodemanipulation.annotated_std import CONSTANT_BUILTINS
 
 
@@ -19,12 +19,17 @@ INIT_ASSEMBLY = True
 
 
 def load_opcode_data():
+    if load_opcode_data.loaded:
+        return
+
+    load_opcode_data.loaded = True
+
     opcode_data: dict = json.load(open(folder + "/opcodes.json"))
 
     valid_opcode_names = [
         key
         for key, value in Opcodes.__dict__.items()
-        if isinstance(value, int) and not key.startswith("__") and value < 256
+        if isinstance(value, int) and not key.startswith("__") and value != -1
     ]
 
     for key, opcode in opcode_data.items():
@@ -37,6 +42,15 @@ def load_opcode_data():
         if key not in opcode_data:
             setattr(Opcodes, key, -1)
 
+    virtual_opcode = 256
+    for key, value in Opcodes.__dict__.items():
+        if not key.startswith("__") and value == -1:
+            setattr(Opcodes, key, virtual_opcode)
+            virtual_opcode += 1
+
+
+load_opcode_data.loaded = False
+
 
 def load_instruction_spec():
     spec_data = json.load(open(folder + "/instruction_spec.json"))
