@@ -1,7 +1,7 @@
 import dis
 
 from bytecodemanipulation import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
 from bytecodemanipulation.Optimiser import (
     guarantee_builtin_names_are_protected,
     guarantee_module_import,
