@@ -3,10 +3,10 @@ import typing
 from bytecodemanipulation.annotated_std import CONSTANT_BUILTIN_TYPES
 from bytecodemanipulation.annotated_std import CONSTANT_BUILTINS
 
-from bytecodemanipulation.MutableFunction import Instruction
+from bytecodemanipulation.opcodes.Instruction import Instruction
 from bytecodemanipulation.MutableFunction import MutableFunction
-from bytecodemanipulation.Opcodes import Opcodes
-from bytecodemanipulation.Opcodes import SIDE_EFFECT_FREE_LOADS
+from bytecodemanipulation.opcodes.Opcodes import Opcodes
+from bytecodemanipulation.opcodes.Opcodes import SIDE_EFFECT_FREE_LOADS
 from bytecodemanipulation.Specialization import ArgDescriptor
 from bytecodemanipulation.Specialization import MethodCallDescriptor
 from bytecodemanipulation.Specialization import SpecializationContainer
@@ -46,34 +46,26 @@ OPCODE_TO_ATTR_DOUBLE = {
     Opcodes.BINARY_OR: "__or__",
     (Opcodes.IS_OP, 0): lambda a, b: a is b,
     (Opcodes.IS_OP, 1): lambda a, b: a is not b,
-    (Opcodes.COMPARE_OP, 0): lambda a, b: a < b,
-    (Opcodes.COMPARE_OP, 1): lambda a, b: a <= b,
-    (Opcodes.COMPARE_OP, 2): lambda a, b: a == b,
-    (Opcodes.COMPARE_OP, 3): lambda a, b: a != b,
-    (Opcodes.COMPARE_OP, 4): lambda a, b: a > b,
-    (Opcodes.COMPARE_OP, 5): lambda a, b: a >= b,
+    Opcodes.COMPARE_LT: lambda a, b: a < b,
+    Opcodes.COMPARE_LE: lambda a, b: a <= b,
+    Opcodes.COMPARE_EQ: lambda a, b: a == b,
+    Opcodes.COMPARE_NEQ: lambda a, b: a != b,
+    Opcodes.COMPARE_GT: lambda a, b: a > b,
+    Opcodes.COMPARE_GE: lambda a, b: a >= b,
 }
 
 
-def inline_const_value_pop_pairs(mutable: MutableFunction) -> bool:
+def inline_const_value_pop_pairs(mutable: MutableFunction, builder) -> bool:
     dirty = False
 
-    for instruction in mutable.instructions:
+    def visit(instruction: Instruction):
+        nonlocal dirty
+
         if instruction.opcode == Opcodes.POP_TOP:
             try:
                 # print("search")
                 source = next(instruction.trace_stack_position(0))
             except StopIteration:
-                # source = next(instruction.trace_stack_position(0))
-
-                print("---")
-
-                for instr in mutable.instructions:
-                    print(instr)
-
-                print("---")
-                print(instruction)
-                print(instruction.get_priorities_previous())
                 raise
 
             # Inline LOAD_XX - POP pairs
@@ -99,23 +91,21 @@ def inline_const_value_pop_pairs(mutable: MutableFunction) -> bool:
 
                         if func_resolve.arg == 0:
                             func_resolve.change_opcode(Opcodes.NOP)
-                            continue
+                            return
+
                         pops = func_resolve.arg
 
                         func_resolve.change_opcode(Opcodes.POP_TOP)
                         pops -= 1
 
                         if not pops:
-                            continue
+                            return
 
                         for _ in range(pops):
-                            nop = Instruction.create(Opcodes.NOP)
+                            nop = Instruction.create_with_same_info(func_resolve, Opcodes.NOP)
                             nop.next_instruction = func_resolve.next_instruction
                             func_resolve.next_instruction = nop
 
-                        mutable.assemble_instructions_from_tree(
-                            mutable.instructions[0].optimise_tree()
-                        )
                         return True
 
             if source.opcode in (
@@ -133,55 +123,57 @@ def inline_const_value_pop_pairs(mutable: MutableFunction) -> bool:
                 if count == 0:
                     source.change_opcode(Opcodes.NOP)
                     dirty = True
-                    continue
+                    return
 
                 source.change_opcode(Opcodes.POP_TOP)
 
                 if count == 1:
                     dirty = True
-                    continue
+                    return
 
                 count -= 1
 
                 for _ in range(count):
-                    nop = Instruction.create(Opcodes.NOP)
+                    nop = Instruction.create_with_same_info(source, Opcodes.NOP)
                     nop.next_instruction = source.next_instruction
                     source.next_instruction = nop
 
-                mutable.assemble_instructions_from_tree(
-                    mutable.instructions[0].optimise_tree()
-                )
-                return True
+    mutable.walk_instructions(visit)
 
     return dirty
 
 
-def remove_local_var_assign_without_use(mutable: MutableFunction) -> bool:
+def remove_local_var_assign_without_use(mutable: MutableFunction, builder) -> bool:
     dirty = False
 
-    last_loads_of_local = [-1] * len(mutable.shared_variable_names)
+    last_loads_of_local = {}
 
-    for instruction in mutable.instructions:
+    def fill_info(instruction: Instruction):
         if instruction.opcode == Opcodes.LOAD_FAST:
-            last_loads_of_local[instruction.arg] = instruction.offset
+            last_loads_of_local[instruction.arg_value] = instruction.offset
 
-    for instruction in mutable.instructions:
+    def remove_instruction(instruction: Instruction):
         if (
             instruction.opcode in (Opcodes.STORE_FAST, Opcodes.DELETE_FAST)
-            and last_loads_of_local[instruction.arg] < instruction.offset
+            and last_loads_of_local.get(instruction.arg_value, -1) < instruction.offset
         ):
             instruction.change_opcode(Opcodes.POP_TOP)
+
+            nonlocal dirty
             dirty = True
 
+    mutable.walk_instructions(fill_info)
+    mutable.walk_instructions(remove_instruction)
+
     return dirty
 
 
-def inline_constant_method_invokes(mutable: MutableFunction) -> bool:
+def inline_constant_method_invokes(mutable: MutableFunction, builder) -> bool:
     dirty = False
 
-    for instr in mutable.instructions:
-        if instr.opcode in (Opcodes.CALL_FUNCTION, Opcodes.CALL_METHOD):
-            target = next(instr.trace_stack_position(instr.arg))
+    def visit(instruction: Instruction):
+        if instruction.opcode in (Opcodes.CALL_FUNCTION, Opcodes.CALL_METHOD):
+            target = next(instruction.trace_stack_position(instruction.arg))
 
             if target.opcode == Opcodes.LOAD_CONST:
                 function: typing.Callable = target.arg_value
@@ -191,29 +183,35 @@ def inline_constant_method_invokes(mutable: MutableFunction) -> bool:
                     and getattr(function, "_OPTIMISER_CONTAINER").is_constant_op
                 ):
                     args = [
-                        next(instr.trace_stack_position(i))
-                        for i in range(instr.arg - 1, -1, -1)
+                        next(instruction.trace_stack_position(i))
+                        for i in range(instruction.arg - 1, -1, -1)
                     ]
 
                     if all(ins.opcode == Opcodes.LOAD_CONST for ins in args):
                         result = function(*(e.arg_value for e in args))
 
-                        instr.change_opcode(Opcodes.LOAD_CONST)
-                        instr.change_arg_value(result)
+                        instruction.change_opcode(Opcodes.LOAD_CONST)
+                        instruction.change_arg_value(result)
                         target.change_opcode(Opcodes.NOP)
 
                         for arg in args:
                             arg.change_opcode(Opcodes.NOP)
 
+                        nonlocal dirty
                         dirty = True
 
+    mutable.walk_instructions(visit)
+
     return dirty
 
 
-def inline_constant_binary_ops(mutable: MutableFunction) -> bool:
+def inline_constant_binary_ops(mutable: MutableFunction, builder) -> bool:
     dirty = False
 
-    for instruction in mutable.instructions:
+    def visit(instruction: Instruction):
+        print(instruction)
+        nonlocal dirty
+
         if (
             instruction.opcode in OPCODE_TO_ATTR_SINGLE
             or (instruction.opcode, instruction.arg) in OPCODE_TO_ATTR_SINGLE
@@ -241,12 +239,13 @@ def inline_constant_binary_ops(mutable: MutableFunction) -> bool:
                             and getattr(method, "_OPTIMISER_CONTAINER").is_constant_op
                         )
                     ):
-                        continue
+                        return
 
                     try:
                         value = method()
                     except:
-                        continue
+                        traceback.print_exc()
+                        return
 
                     instruction.change_opcode(Opcodes.LOAD_CONST)
                     instruction.change_arg_value(value)
@@ -263,14 +262,20 @@ def inline_constant_binary_ops(mutable: MutableFunction) -> bool:
                 else (instruction.opcode, instruction.arg)
             ]
 
-            arg = next(instruction.trace_stack_position(1))
-            target = next(instruction.trace_stack_position(0))
+            target = next(instruction.trace_stack_position(1))
+            arg = next(instruction.trace_stack_position(0))
 
             if arg.opcode == target.opcode == Opcodes.LOAD_CONST:
                 value = target.arg_value
 
+                add_target = True
+
                 if isinstance(method, str):
+                    if not hasattr(value, method):
+                        return
+
                     method = getattr(value, method)
+                    add_target = False
 
                     if not callable(method) or not (
                         type(value) in CONSTANT_BUILTIN_TYPES
@@ -279,13 +284,16 @@ def inline_constant_binary_ops(mutable: MutableFunction) -> bool:
                             and getattr(method, "_OPTIMISER_CONTAINER").is_constant_op
                         )
                     ):
-                        continue
+                        return
 
                 try:
-                    value = method(arg.arg_value, target.arg_value)
+                    if add_target:
+                        value = method(arg.arg_value, target.arg_value)
+                    else:
+                        value = method(arg.arg_value)
                 except:
                     traceback.print_exc()
-                    continue
+                    return
 
                 instruction.change_opcode(Opcodes.LOAD_CONST)
                 instruction.change_arg_value(value)
@@ -338,13 +346,17 @@ def inline_constant_binary_ops(mutable: MutableFunction) -> bool:
 
                 dirty = True
 
+    mutable.walk_instructions(visit)
+
     return dirty
 
 
-def remove_branch_on_constant(mutable: MutableFunction) -> bool:
+def remove_branch_on_constant(mutable: MutableFunction, builder) -> bool:
     dirty = False
 
-    for instruction in mutable.instructions:
+    def visit(instruction):
+        nonlocal dirty
+
         if (
             instruction.has_jump()
             and not instruction.has_unconditional_jump()
@@ -355,7 +367,7 @@ def remove_branch_on_constant(mutable: MutableFunction) -> bool:
                     instruction.offset == 0
                     and instruction.opcode == Opcodes.SETUP_FINALLY
                 ):
-                    continue
+                    return
 
                 raise RuntimeError
 
@@ -379,7 +391,7 @@ def remove_branch_on_constant(mutable: MutableFunction) -> bool:
 
                     dirty = True
 
-                    continue
+                    return
 
                 if instruction.opcode in (
                     Opcodes.POP_JUMP_IF_TRUE,
@@ -399,13 +411,15 @@ def remove_branch_on_constant(mutable: MutableFunction) -> bool:
 
                     dirty = True
 
+    mutable.walk_instructions(visit)
+
     return dirty
 
 
-def inline_static_attribute_access(mutable: MutableFunction) -> bool:
+def inline_static_attribute_access(mutable: MutableFunction, builder) -> bool:
     dirty = False
 
-    for instruction in mutable.instructions:
+    def visit(instruction: Instruction):
         if instruction.opcode in (Opcodes.LOAD_ATTR, Opcodes.LOAD_METHOD):
             source_instr = next(instruction.trace_stack_position(0))
 
@@ -429,17 +443,22 @@ def inline_static_attribute_access(mutable: MutableFunction) -> bool:
                     if use.opcode == Opcodes.CALL_METHOD:
                         use.change_opcode(Opcodes.CALL_FUNCTION)
 
+                    nonlocal dirty
                     dirty = True
 
+    mutable.walk_instructions(visit)
+
     return dirty
 
 
-def apply_specializations(mutable: MutableFunction) -> bool:
+def apply_specializations(mutable: MutableFunction, builder) -> bool:
     from bytecodemanipulation.Optimiser import _OptimisationContainer
 
     dirty = False
 
-    for instruction in mutable.instructions:
+    def visit(instruction: Instruction):
+        nonlocal dirty
+
         if instruction.opcode in (Opcodes.CALL_FUNCTION, Opcodes.CALL_METHOD):
             load_stack_pos = instruction.arg
             source = instruction.trace_normalized_stack_position(load_stack_pos)
@@ -449,7 +468,7 @@ def apply_specializations(mutable: MutableFunction) -> bool:
                 container = _OptimisationContainer.get_for_target(source.arg_value)
 
                 if not container.specializations:
-                    continue
+                    return
 
                 target_func = source.arg_value
 
@@ -480,6 +499,8 @@ def apply_specializations(mutable: MutableFunction) -> bool:
                     dirty = True
                     break
 
+    mutable.walk_instructions(visit)
+
     return dirty
 
 
