@@ -1,6 +1,7 @@
 import dis
 import functools
 import itertools
+import sys
 from unittest import TestCase
 
 import bytecodemanipulation.data_loader
@@ -45,10 +46,6 @@ from bytecodemanipulation.data.shared.instructions.WhileAssembly import (
 from bytecodemanipulation.data.shared.instructions.YieldAssembly import (
     AbstractYieldAssembly,
 )
-from bytecodemanipulation.data.shared.instructions.OpAssembly import AbstractOpAssembly
-from bytecodemanipulation.data.shared.instructions.CallAssembly import (
-    AbstractCallAssembly,
-)
 from bytecodemanipulation.MutableFunction import MutableFunction
 
 bytecodemanipulation.data_loader.INIT_ASSEMBLY = False
@@ -60,6 +57,8 @@ from bytecodemanipulation.assembler.target import (
     make_macro,
 )
 from bytecodemanipulation.assembler.Emitter import apply_inline_assemblies
+from bytecodemanipulation.MutableFunctionHelpers import outer_return
+from bytecodemanipulation.MutableFunctionHelpers import capture_local
 
 try:
     from code_parser.parsers.common import IdentifierExpression
@@ -150,8 +149,8 @@ LOAD 19 --> $test
         self.assertEqualList(
             CompoundExpression(
                 [
-                    AbstractStoreGlobalAssembly.IMPLEMENTATION(IdentifierToken("test")),
-                    AbstractStoreGlobalAssembly.IMPLEMENTATION(IdentifierToken("test")),
+                    AbstractStoreGlobalAssembly.IMPLEMENTATION("test"),
+                    AbstractStoreGlobalAssembly.IMPLEMENTATION("test"),
                     AbstractStoreGlobalAssembly.IMPLEMENTATION(
                         "test",
                         GlobalAccessExpression("test"),
@@ -723,15 +722,15 @@ DEF test <test> () -> @target {}
                     AbstractFunctionDefinitionAssembly.IMPLEMENTATION(
                         "test",
                         ["test"],
-                        [AbstractCallAssembly.Arg(IdentifierToken("a"))],
+                        [AbstractCallAssembly.Arg("a")],
                         CompoundExpression([]),
                     ),
                     AbstractFunctionDefinitionAssembly.IMPLEMENTATION(
                         "test",
                         ["test"],
                         [
-                            AbstractCallAssembly.Arg(IdentifierToken("a")),
-                            AbstractCallAssembly.Arg(IdentifierToken("b")),
+                            AbstractCallAssembly.Arg("a"),
+                            AbstractCallAssembly.Arg("b"),
                         ],
                         CompoundExpression([]),
                     ),
@@ -913,6 +912,7 @@ YIELD @GLOBAL -> $v"""
 CALL PARTIAL @print ("Hello World!") -> $x
 """
             )
+            return x
 
         mutable = MutableFunction(target)
         apply_inline_assemblies(mutable)
@@ -921,6 +921,7 @@ CALL PARTIAL @print ("Hello World!") -> $x
         @cache_global_name("functools")
         def compare():
             x = functools.partial(print, "Hello World!")
+            return x
 
         compare_optimized_results(self, target, compare, opt_ideal=2)
 
@@ -939,6 +940,8 @@ CALL @print ("Hello World!") -> $x
         apply_inline_assemblies(mutable)
         mutable.reassign_to_function()
 
+        dis.dis(target)
+
         target(self)
 
         def compare():
@@ -976,10 +979,32 @@ PYTHON {
         bytecodemanipulation.assembler.hook.unhook()
 
     def test_dynamic_attribute_access(self):
+        @apply_operations
         def target(t):
             assembly(
                 """
-RETURN $t.("test_dynamic_attribute_access")
+RETURN $t.("attr")
+"""
+            )
+
+        class H:
+            attr = {}
+
+        x = H()
+
+        self.assertEqual(target(x), x.attr)
+
+    def test_static_attribute_access(self):
+        class H:
+            attr = {}
+
+        global x
+        x = H()
+
+        def target():
+            assembly(
+                """
+RETURN @!x.!attr
 """
             )
 
@@ -987,7 +1012,14 @@ RETURN $t.("test_dynamic_attribute_access")
         apply_inline_assemblies(mutable)
         mutable.reassign_to_function()
 
-        self.assertEqual(target(self), self.test_dynamic_attribute_access)
+        dis.dis(target)
+
+        value = x.attr
+        x.attr = None
+        self.assertEqual(target(), value)
+
+        x = None
+        self.assertEqual(target(), value)
 
 
 class TestMacro(TestCase):
@@ -997,7 +1029,7 @@ class TestMacro(TestCase):
                 """
     LOAD 1 -> $x
     MACRO test_basic {
-        LOAD 0 -> $x
+        LOAD 0 -> $:x
     }
     
     CALL MACRO test_basic()
@@ -1017,7 +1049,7 @@ class TestMacro(TestCase):
             assembly(
                 """
         MACRO test_macro_local_access {
-            LOAD 1 -> $local
+            LOAD 1 -> $:local
         }
 
         LOAD 0 -> $local
@@ -1038,7 +1070,7 @@ class TestMacro(TestCase):
             assembly(
                 """
         MACRO test_macro_parameter_resolver (!param) {
-            LOAD &param -> $local
+            LOAD &param -> $:local
             LOAD 0 -> &param
         }
 
@@ -1056,13 +1088,14 @@ class TestMacro(TestCase):
         self.assertEqual(target(), 1)
 
     def test_macro_parameter_duplicated_access_static(self):
+        @apply_operations
         def target():
             assembly(
                 """
         MACRO test_macro_parameter_duplicated_access_static (!param) {
-            LOAD &param -> $local
+            LOAD &param -> $:local
             LOAD 2 -> &param
-            LOAD &param -> $local
+            LOAD &param -> $:local
         }
 
         LOAD 0 -> $local
@@ -1072,9 +1105,7 @@ class TestMacro(TestCase):
             )
             return -1
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
+        dis.dis(target)
 
         self.assertEqual(target(), 2)
 
@@ -1100,7 +1131,7 @@ class TestMacro(TestCase):
             assembly(
                 """
         MACRO test_macro_local_name_override (!param) {
-            LOAD 10 -> $test
+            LOAD 10 -> $:test
         }
 
         LOAD 0 -> $test
@@ -1121,7 +1152,7 @@ class TestMacro(TestCase):
             assembly(
                 """
         MACRO test_macro_local_name_escape (!param) {
-            LOAD 10 -> $MACRO_test
+            LOAD 10 -> $test
         }
 
         LOAD 0 -> $test
@@ -1224,15 +1255,15 @@ class TestMacro(TestCase):
         def target():
             assembly(
                 """
-        MACRO test_macro_paste (param) {
-            MACRO_PASTE param
-        }
+MACRO test_macro_paste (param) {
+    MACRO_PASTE param
+}
 
-        LOAD 0 -> $test
-        CALL MACRO test_macro_paste({
-            LOAD 10 -> $test
-        })
-        RETURN $test
+LOAD 0 -> $test
+CALL MACRO test_macro_paste({
+    LOAD 10 -> $test
+})
+RETURN $test
         """
             )
             return -1
@@ -1285,11 +1316,12 @@ class TestMacro(TestCase):
         self.assertRaises(NameError, lambda: apply_inline_assemblies(mutable))
 
     def test_macro_paste_use(self):
+        @apply_operations
         def target():
             assembly(
                 """
         MACRO test_macro_paste_use (param) {
-            MACRO_PASTE param -> $test
+            MACRO_PASTE param -> $:test
         }
 
         LOAD 0 -> $test
@@ -1301,9 +1333,27 @@ class TestMacro(TestCase):
             )
             return -1
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
+        self.assertEqual(target(), 10)
+
+    def test_macro_paste_use_vbar_prefix(self):
+        @apply_operations
+        def target():
+            assembly(
+                """
+        MACRO test_macro_paste_use_vbar (param) {
+            LOAD 0 -> $test
+            
+            MACRO_PASTE param
+            
+            RETURN $test
+        }
+
+        CALL MACRO test_macro_paste_use_vbar({
+            LOAD 10 -> $|test
+        })
+        """
+            )
+            return -1
 
         self.assertEqual(target(), 10)
 
@@ -1379,6 +1429,161 @@ class TestMacro(TestCase):
 
         self.assertEqual(target(), 1)
 
+    def test_macro_capture_arg_in_inner_func(self):
+        def target():
+            tar = lambda: 0
+            assembly(
+                """
+MACRO test_macro_capture_arg_in_inner_func(a)
+{
+    DEF :tar()
+    {
+        RETURN &a
+    }
+}
+
+CALL MACRO test_macro_capture_arg_in_inner_func(2)
+"""
+            )
+            return tar
+
+        mutable = MutableFunction(target)
+        apply_inline_assemblies(mutable)
+        mutable.reassign_to_function()
+
+        # dis.dis(target)
+
+        self.assertEqual(target()(), 2)
+
+    def test_macro_capture_arg_in_inner_func_2(self):
+        def target():
+            tar = lambda: 0
+            assembly(
+                """
+MACRO test_macro_capture_arg_in_inner_func_2(a CODE_BLOCK)
+{
+    DEF :tar()
+    {
+        MACRO_PASTE a
+    }
+}
+
+CALL MACRO test_macro_capture_arg_in_inner_func_2({ RETURN 2 })
+"""
+            )
+            return tar
+
+        mutable = MutableFunction(target)
+        apply_inline_assemblies(mutable)
+        mutable.reassign_to_function()
+
+        # dis.dis(target)
+
+        self.assertEqual(target()(), 2)
+
+    def test_macro_func_definition_keyword_expansion(self):
+        @apply_operations
+        def target():
+            assembly("""
+MACRO test_macro_func_definition_keyword_expansion(keyword) -> ANY
+{
+    DEF export(&keyword=0)
+    {
+        RETURN $&keyword
+    }
+    
+    MACRO_RETURN $export
+}
+
+CALL MACRO test_macro_func_definition_keyword_expansion("key") -> $func
+RETURN $func
+""")
+            return 0
+
+        inter = target()
+
+        self.assertEqual(inter.__defaults__, (0,))
+
+        self.assertEqual(inter(key=10), 10)
+
+    def test_macro_func_call_keyword_expansion(self):
+        @apply_operations
+        def target():
+            assembly("""
+
+DEF xy(a=0)
+{
+    RETURN $a
+}            
+
+MACRO test_macro_func_call_keyword_expansion(keyword)
+{
+    RETURN $:xy(&keyword=4)
+}
+
+CALL MACRO test_macro_func_call_keyword_expansion("a")
+""")
+            return 0
+
+        self.assertEqual(target(), 4)
+
+    def test_macro_paste_parameterized_code_block_1(self):
+        @apply_operations
+        def target():
+            assembly("""    
+
+MACRO test_macro_paste_parameterized_code_block_1(code CODE_BLOCK[1])
+{
+    LOAD 10 -> $var
+    MACRO_PASTE &code [$var]
+}
+
+CALL MACRO test_macro_paste_parameterized_code_block_1([$local] { RETURN $local })
+        """)
+            return 0
+
+        self.assertEqual(target(), 10)
+
+    def test_macro_paste_parameterized_code_block_non_static(self):
+        @apply_operations
+        def target():
+            assembly("""    
+
+MACRO test_macro_paste_parameterized_code_block_non_static(code CODE_BLOCK[1])
+{
+    LOAD 10 -> $var
+    MACRO_PASTE &code [$var]
+}
+
+CALL MACRO test_macro_paste_parameterized_code_block_non_static([$local] { LOAD 0 -> $|var\nRETURN $local })
+        """)
+            return -1
+
+        dis.dis(target)
+
+        self.assertEqual(target(), 0)
+
+    def test_macro_paste_parameterized_code_block_static(self):
+        @apply_operations
+        def target():
+            assembly("""    
+
+MACRO test_macro_paste_parameterized_code_block_static(code CODE_BLOCK[1])
+{
+    LOAD 10 -> $var
+    MACRO_PASTE &code [!$var]
+}
+
+CALL MACRO test_macro_paste_parameterized_code_block_static([$local] { LOAD 0 -> $|var\nRETURN $local })
+        """)
+            return 0
+
+        dis.dis(target)
+
+        self.assertEqual(target(), 10)
+
+
+class TestClassAssembly(TestCase):
     def test_class_assembly(self):
         def target():
             test = None
@@ -1454,11 +1659,15 @@ CALL MACRO test:test_macro()
 
         self.assertEqual(target(), 1)
 
+
+class TestFunc2Macro(TestCase):
     def test_transform_to_macro(self):
         @make_macro("TestMacro:test_transform_to_macro")
         def test_transform_to_macro():
+            capture_local("test")
             test = 1
 
+        @apply_operations
         def target():
             test = -1
             assembly(
@@ -1469,17 +1678,15 @@ CALL MACRO TestMacro:test_transform_to_macro()
             )
             return test
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
         self.assertEqual(target(), 1)
 
     def test_transform_to_macro_2(self):
         @make_macro("TestMacro:test_transform_to_macro_2")
         def test_transform_to_macro(a):
+            capture_local("test")
             test = a
 
+        @apply_operations
         def target():
             test = -1
             assembly(
@@ -1490,17 +1697,15 @@ CALL MACRO TestMacro:test_transform_to_macro_2(1)
             )
             return test
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
         self.assertEqual(target(), 1)
 
     def test_transform_to_macro_3(self):
         @make_macro("TestMacro:test_transform_to_macro_3")
         def test_transform_to_macro(a):
+            capture_local("test")
             test = a + a
 
+        @apply_operations
         def target():
             test = -1
             value = [1, 2]
@@ -1512,10 +1717,6 @@ CALL MACRO TestMacro:test_transform_to_macro_3({ CALL $value.pop() })
             )
             return test
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
         self.assertEqual(target(), 3)
 
     def test_transform_to_macro_or_else_error(self):
@@ -1527,58 +1728,24 @@ CALL MACRO TestMacro:test_transform_to_macro_3({ CALL $value.pop() })
 
         self.assertRaises(RuntimeError, test_transform_to_macro)
 
-    def test_macro_capture_arg_in_inner_func(self):
+    def test_transform_to_macro_inner_return(self):
+        @make_macro("TestMacro:test_transform_to_macro_inner_return")
+        def test_transform_to_macro():
+            outer_return(1)
+
+        @apply_operations
         def target():
-            tar = lambda: 0
             assembly(
                 """
-MACRO test_macro_capture_arg_in_inner_func(a)
-{
-    DEF tar()
-    {
-        RETURN &a
-    }
-}
-
-CALL MACRO test_macro_capture_arg_in_inner_func(2)
+CALL MACRO TestMacro:test_transform_to_macro_inner_return()
 """
             )
-            return tar
+            return 0
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
+        self.assertEqual(target(), 1)
 
-        # dis.dis(target)
-
-        self.assertEqual(target()(), 2)
-
-    def test_macro_capture_arg_in_inner_func_2(self):
-        def target():
-            tar = lambda: 0
-            assembly(
-                """
-MACRO test_macro_capture_arg_in_inner_func_2(a CODE_BLOCK)
-{
-    DEF tar()
-    {
-        MACRO_PASTE a
-    }
-}
-
-CALL MACRO test_macro_capture_arg_in_inner_func_2({ RETURN 2 })
-"""
-            )
-            return tar
-
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        # dis.dis(target)
-
-        self.assertEqual(target()(), 2)
 
+class TestForLoopAssembly(TestCase):
     def test_for_loop_basic(self):
         def target():
             iterable = [0, 1, 2, 3]
@@ -1597,7 +1764,7 @@ FOREACH $p IN $iterable
         apply_inline_assemblies(mutable)
         mutable.reassign_to_function()
 
-        # dis.dis(target)
+        dis.dis(target)
 
         self.assertEqual(target(), [1, 2, 3, 4])
 
@@ -1643,6 +1810,8 @@ FOREACH $p IN $iterable * $iterable_2
 
         self.assertEqual(target(), [0, 0, 1, 2])
 
+
+class TestRawAssembly(TestCase):
     def test_raw_assembly_by_id(self):
         def target():
             assembly(
@@ -1677,6 +1846,8 @@ RETURN %
 
         self.assertEqual(target(), 10)
 
+
+class TestAssert(TestCase):
     def test_assert(self):
         @apply_operations
         def target(x):
@@ -1686,6 +1857,8 @@ ASSERT $x
 """
             )
 
+        dis.dis(target)
+
         target(1)
         self.assertRaises(AssertionError, lambda: target(0))
 
@@ -1707,3 +1880,178 @@ ASSERT $x \"Test Message\"
             self.assertEqual(e.args, ("Test Message",))
         else:
             self.assertTrue(False)
+
+    def test_assert_static(self):
+        @apply_operations
+        def target():
+            assembly("ASSERT_STATIC 1")
+
+        target()
+
+    def test_assert_static_fail(self):
+        def target():
+            assembly("ASSERT_STATIC 0")
+
+        try:
+            apply_operations(target)
+        except AssertionError as e:
+            self.assertEqual(e.args, ("assertion failed: expected <true-ish value>",))
+
+    def test_assert_static_fail_message(self):
+        def target():
+            assembly('ASSERT_STATIC 0 "hello world"')
+
+        try:
+            apply_operations(target)
+        except AssertionError as e:
+            self.assertEqual(e.args, ("assertion failed: hello world",))
+
+    def test_assert_static_fail_message_invalid(self):
+        def target():
+            assembly("""
+ASSERT_STATIC 0 $x""")
+
+        try:
+            apply_operations(target)
+        except AssertionError as e:
+            self.assertEqual(
+                e.args,
+                ("assertion failed: expected <true-ish value> (message not arrival)",),
+            )
+
+    def test_assert_static_dynamic_expression(self):
+        def target():
+            assembly("ASSERT_STATIC $a")
+
+        try:
+            apply_operations(target)
+        except SyntaxError as e:
+            self.assertEqual(e.args, ("Expected <static evaluate-able at 'expression'>",))
+
+    def test_assert_static_macro_static_parameter(self):
+        @apply_operations
+        def target():
+            assembly(
+                """
+MACRO test_assert_static_macro_static_parameter(a)
+{
+    ASSERT_STATIC &a
+}
+
+CALL MACRO test_assert_static_macro_static_parameter(1)
+"""
+            )
+
+        target()
+
+    def test_assert_static_macro_static_parameter_fail(self):
+        def target():
+            assembly(
+                """
+MACRO test_assert_static_macro_static_parameter_fail(a)
+{
+    ASSERT_STATIC &a
+}
+
+CALL MACRO test_assert_static_macro_static_parameter_fail(0)
+"""
+            )
+
+        try:
+            apply_operations(target)
+        except AssertionError as e:
+            self.assertEqual(e.args, ("assertion failed: expected <true-ish value>",))
+
+
+class TestIfLabel(TestCase):
+    def test_if_label_jump(self):
+        @apply_operations
+        def target(x):
+            assembly("""
+
+IF $x 'label' 
+{
+    JUMP label:END
+    RETURN 0
+}
+
+RETURN 1
+""")
+
+        self.assertEqual(target(0), 1)
+        self.assertEqual(target(1), 1)
+
+    def test_version_constants(self):
+        @apply_operations
+        def target():
+            assembly("""
+
+RETURN ~PY_VERSION
+        """)
+
+        self.assertEqual(target(), sys.version_info.major * 100 + sys.version_info.minor)
+
+
+class TestStaticIf(TestCase):
+    def test_static_version_check(self):
+        version_id = sys.version_info.major * 100 + sys.version_info.minor
+
+        scope = {"__file__": __file__, "apply_operations": apply_operations, "assembly": assembly}
+
+        exec(f'''
+@apply_operations
+def target():
+    assembly("""
+IF OP(~PY_VERSION == {version_id})
+{{
+    RETURN 1
+}}
+RETURN 0
+""")''', scope)
+
+        target = scope["target"]
+
+        self.assertEqual(target(), 1)
+        compare_optimized_results(self, target, lambda: 1)
+
+
+class TestFunctionDefinitionLocalCapture(TestCase):
+    def test_simple(self):
+        @apply_operations
+        def target():
+            assembly("""
+LOAD 10 -> $test
+
+DEF func<test>()
+{
+    RETURN §test
+}
+
+RETURN $func()
+""")
+
+        self.assertEqual(target(), 10)
+
+    def test_write(self):
+        @apply_operations
+        def target():
+            assembly("""
+LOAD 0 -> $test            
+
+DEF func<test>()
+{
+    LOAD 10 -> §test
+    RETURN §test
+}
+
+CALL ~print(10) -> \\
+
+LOAD $func() -> $result
+# ASSERT OP ($test == 10)  # wont work as we currently cannot write back outside
+ASSERT OP ($test == 0) "old implementation limitation lifted?"
+RETURN $result 
+""")
+
+        dis.dis(target)
+
+        self.assertEqual(target(), 10)
