@@ -48,7 +48,7 @@ class StandardLibraryTest(TestCase):
 
     def test_type_check_raise(self):
         def target():
-            assembly("""std:check_type(@int, "test", "exception")""")
+            assembly("""std:check_type(~int, "test", "exception")""")
 
         mutable = MutableFunction(target)
         apply_inline_assemblies(mutable)
@@ -125,7 +125,7 @@ std:stream:to_list($stream, $output)
                 """
 std:stream:initialize($stream)
 std:stream:extend($stream, $data)
-std:stream:reduce($stream, $lhs, $rhs, {
+std:stream:reduce($stream, [$lhs, $rhs] {
     OP $lhs + $rhs
 })
 STORE $output
@@ -140,6 +140,8 @@ STORE $output
         self.assertEqual(target(), 3)
 
     def test_stream_simple_filter(self):
+
+        @apply_operations
         def target():
             data = (0, 1, 2)
             stream = None
@@ -149,7 +151,7 @@ STORE $output
 std:stream:initialize($stream)
 std:stream:extend($stream, $data)
 std:print($stream)
-std:stream:filter($stream, $var, {
+std:stream:filter($stream, [$var] {
     OP $var < 2 -> %
 })
 std:print($stream)
@@ -158,15 +160,10 @@ std:stream:to_list($stream, $output)
             )
             return output
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
-        # dis.dis(target)
-
         self.assertEqual(target(), [0, 1])
 
     def test_stream_simple_map(self):
+        @apply_operations
         def target():
             data = (0, 1, 2)
             stream = None
@@ -175,7 +172,7 @@ std:stream:to_list($stream, $output)
                 """
 std:stream:initialize($stream)
 std:stream:extend($stream, $data)
-std:stream:map($stream, $var, {
+std:stream:map($stream, [$var] {
     OP $var + 1 -> $var
 })
 std:stream:to_list($stream, $output)
@@ -183,24 +180,14 @@ std:stream:to_list($stream, $output)
             )
             return output
 
-        mutable = MutableFunction(target)
-        apply_inline_assemblies(mutable)
-        mutable.reassign_to_function()
-
         self.assertEqual(target(), [1, 2, 3])
 
+    def test_comprehension_list(self):
+        @apply_operations
+        def target():
+            l = [1, 2, 3, 4]
+            assembly("""
+std:comprehension:list($l, [$value] { OP $value + 1 -> % }) -> %
+RETURN %""")
 
-#     def test_comprehension_list(self):
-#         def target():
-#             l = [1, 2, 3, 4]
-#             assembly("""
-# std:comprehension:list($l, $value, { OP $value + 1 -> % }) -> %
-# RETURN %""")
-#
-#         mutable = MutableFunction(target)
-#         apply_inline_assemblies(mutable)
-#         mutable.reassign_to_function()
-#
-#         dis.dis(target)
-#
-#         self.assertEqual(target(), [1, 2, 3])
+        self.assertEqual(target(), [2, 3, 4, 5])
